#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   BaseConfig.py
@Path    :   agrothon/
@Time    :   2021/05/28
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.3.1
@Contact :   ckvbalusu@gmail.com
@Desc    :   Base Configuration for Agrothon
"""
import os

from dotenv import load_dotenv

load_dotenv("agrothon.env")


class Config(object):

    # Telegram
    TELEGRAM_APP_ID = int(os.environ.get("TELEGRAM_APP_ID", -1))
    TELEGRAM_API_HASH = os.environ.get("TELEGRAM_API_HASH", None)
    BOT_TOKEN = os.environ.get("BOT_TOKEN", None)
    API_BASE_URL = os.environ.get("API_BASE_URL", None)
    ALERT_CHANNEL_ID = int(os.environ.get("ALERT_CHANNEL_ID", -1))
    STATE = os.environ.get("STATE", None)
    DISTRICT = os.environ.get("DISTRICT", None)
    DEF_CITY = os.environ.get("DEF_CITY", None)
    LANGUAGE = os.environ.get("DEF_LANG", "english")

    # Bot Commands
    STATS_COMMAND = os.environ.get("STATS_COMMAND", "stats")
    FIELD_COMMAND = os.environ.get("FIELD_COMMAND", "field")
    SETTINGS_COMMAND = os.environ.get("SETTINGS_COMMAND", "settings")
    WEATHER_COMMAND = os.environ.get("WEATHER_COMMAND", "weather")
    LOG_COMMAND = os.environ.get("LOG_COMMAND", "log")
    HELP_COMMAND = os.environ.get("HELP_COMMAND", "help")
    RAIN_COMMAND = os.environ.get("RAIN_COMMAND", "rainfall")
    RESTART_COMMAND = os.environ.get("RESTART_COMMAND", "restart")
    PING_COMMAND = os.environ.get("PING_COMMAND", "ping")

    # API Server
    OPEN_WEATHER_API = os.environ.get("OPEN_WEATHER_API", None)
    API_KEY = os.environ.get("API_KEY", None)
    DB_URL = os.environ.get("DB_URL", None)
    TIME_ZONE = os.environ.get("TIME_ZONE", "Asia/Kolkata")
    try:
        SENSOR_PRIORITY_INDEX = int(os.environ.get("SENSOR_PRIORITY_INDEX", 1))
    except BaseException:
        SENSOR_PRIORITY_INDEX = 1
    try:
        AUTH_ID = int(os.environ.get("AUTH_ID", None))
    except BaseException:
        AUTH_ID = None



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   api_key_helper.py
@Path    :   agrothon/server/helpers/
@Time    :   2021/05/6
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.3
@Contact :   ckvbalusu@gmail.com
@Desc    :   API Key auth Checker Module
"""
import logging

from fastapi import HTTPException, Security, status
from fastapi.security.api_key import APIKeyQuery

from agrothon import API_KEY

API_KEY_NAME = "api_key"
LOGGER = logging.getLogger(__name__)

api_key_query_auth = APIKeyQuery(name=API_KEY_NAME, auto_error=True)


async def verify_api_key(api_key_query: str = Security(api_key_query_auth)):
    if api_key_query != API_KEY:
        LOGGER.info("API Key Mismatch")
        LOGGER.debug(f"Wrong API Key used : {api_key_query}")
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Unauthorized",
        )



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   __main__.py
@Path    :   agrothon/
@Time    :   2021/05/28
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.3.1
@Contact :   ckvbalusu@gmail.com
@Desc    :   Main Module for Agrothon
"""
import logging
from threading import Thread

from pyrogram import Client, filters
from pyrogram.handlers import CallbackQueryHandler, MessageHandler

from agrothon import (
    AUTH_ID,
    BOT_TOKEN,
    FIELD_COMMAND,
    HELP_COMMAND,
    LOG_COMMAND,
    LOGGER,
    PING_COMMAND,
    RAIN_COMMAND,
    RESTART_COMMAND,
    SETTINGS_COMMAND,
    STATS_COMMAND,
    TELEGRAM_API_HASH,
    TELEGRAM_APP_ID,
    WEATHER_COMMAND,
)

from .AlertBot import alerts_handler, language_change_check, restart_check
from .tgbot.modules.callbacks import (
    backcbq,
    callback_sensors,
    lang_change,
    languages,
    pumpque,
    restart_callback,
)
from .tgbot.modules.fieldstatus import field
from .tgbot.modules.photo_handler import photo_detect
from .tgbot.modules.rainfall import rainfall_predict
from .tgbot.modules.settings import help_command, ping_command, restart, settings, start
from .tgbot.modules.utils import send_log, stats
from .tgbot.modules.weather import weather

logging.getLogger("pyrogram").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)
logging.getLogger("telegram").setLevel(logging.INFO)

AgroBot = Client(
    "Agrothon",
    bot_token=BOT_TOKEN,
    api_id=TELEGRAM_APP_ID,
    api_hash=TELEGRAM_API_HASH,
    workers=20,
    device_model="Agrothon",
)


AUTH_FILTER = None

if AUTH_ID is not None:
    AUTH_FILTER = filters.user(users=AUTH_ID)
else:
    AUTH_FILTER = filters.all


# Callback Query handlers
cb_sensors = CallbackQueryHandler(
    callback_sensors,
    filters=filters.regex(pattern="^moisture|humidity|temperature|complete$"),
)
AgroBot.add_handler(cb_sensors)

back_cbq = CallbackQueryHandler(backcbq, filters=filters.regex(pattern="^back|exit$"))
AgroBot.add_handler(back_cbq)

pump_que = CallbackQueryHandler(
    pumpque, filters=filters.regex(pattern="^pumpstat|pumpon|pumpoff|refresh|bot$")
)
AgroBot.add_handler(pump_que)

languages_ = CallbackQueryHandler(
    languages, filters=filters.regex(pattern="^eng|tel|tam|hin$")
)
AgroBot.add_handler(languages_)

lang_hand = CallbackQueryHandler(lang_change, filters=filters.regex(pattern="^lang$"))
AgroBot.add_handler(lang_hand)

rsrt_hand = CallbackQueryHandler(
    restart_callback, filters=filters.regex(pattern="^restart$")
)
AgroBot.add_handler(rsrt_hand)

# Command handlers
field_hand = MessageHandler(
    field, filters=filters.command([FIELD_COMMAND]) & filters.private & AUTH_FILTER
)
AgroBot.add_handler(field_hand)

phot_detect = MessageHandler(
    photo_detect, filters=filters.photo & filters.private & AUTH_FILTER
)
AgroBot.add_handler(phot_detect)

rain_hand = MessageHandler(
    rainfall_predict,
    filters=filters.command([RAIN_COMMAND]) & filters.private & AUTH_FILTER,
)
AgroBot.add_handler(rain_hand)

set_hand = MessageHandler(
    settings,
    filters=filters.command([SETTINGS_COMMAND]) & filters.private & AUTH_FILTER,
)
AgroBot.add_handler(set_hand)

rest_hand = MessageHandler(
    restart, filters=filters.command([RESTART_COMMAND]) & filters.private & AUTH_FILTER
)
AgroBot.add_handler(rest_hand)

start_hand = MessageHandler(
    start, filters=filters.command(["start"]) & filters.private & AUTH_FILTER
)
AgroBot.add_handler(start_hand)

help_hand = MessageHandler(
    help_command,
    filters=filters.command([HELP_COMMAND]) & filters.private & AUTH_FILTER,
)
AgroBot.add_handler(help_hand)

ping_hand = MessageHandler(
    ping_command,
    filters=filters.command([PING_COMMAND]) & filters.private & AUTH_FILTER,
)
AgroBot.add_handler(ping_hand)

stats_hand = MessageHandler(
    stats, filters=filters.command([STATS_COMMAND]) & filters.private & AUTH_FILTER
)
AgroBot.add_handler(stats_hand)

log_hand = MessageHandler(
    send_log, filters=filters.command([LOG_COMMAND]) & filters.private & AUTH_FILTER
)
AgroBot.add_handler(log_hand)

weather_hand = MessageHandler(
    weather, filters=filters.command([WEATHER_COMMAND]) & filters.private & AUTH_FILTER
)
AgroBot.add_handler(weather_hand)


def main():
    restart_check()
    language_change_check()
    LOGGER.info("Starting Bot")
    Thread(target=alerts_handler, daemon=True).start()
    AgroBot.run()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   openweather.py
@Path    :   agrothon/server/routers/
@Time    :   2021/05/3
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.6
@Contact :   ckvbalusu@gmail.com
@Desc    :   OpenWeather data router
"""

import logging

from fastapi import APIRouter, HTTPException
from fastapi.responses import ORJSONResponse

from ..helpers.openwapi import open_weather_helper_city
from ..helpers.response_models import Weather

LOGGER = logging.getLogger(__name__)

OpenWeatherRouter = APIRouter(
    prefix="/weather",
    tags=["Weather"],
    responses={404: {"error": "Not found"}},
)


@OpenWeatherRouter.get("/{city}", response_class=ORJSONResponse, response_model=Weather)
async def weather_get(city: str):
    """
    Open Weather API Wrapper
    - **city** : City from which the weather data should be fetched (Optional)
    * returns
        * city
        * temperature
        * pressure
        * humidity
        * weather description
    """
    LOGGER.info(f"Getting Weather for {city}")
    status, resp = await open_weather_helper_city(city)
    if status:
        main = resp["main"]
        return ORJSONResponse(
            content={
                "city": resp["name"],
                "temperature": main["temp"],
                "pressure": main["pressure"],
                "humidity": main["humidity"],
                "weather": resp["weather"][0]["description"],
            }
        )
    else:
        raise HTTPException(status_code=404, detail={"error": "API Request Error"})



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   fieldstatus.py
@Path    :   agrothon/tgbot/modules/
@Time    :   2021/05/9
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.5
@Contact :   ckvbalusu@gmail.com
@Desc    :   Field Status Command handler
"""
from agrothon import LANG

from ..helpers.keyboards import fieldkey


async def field(client, message):
    await message.reply_text(text=LANG.MAIN_MENU, reply_markup=fieldkey)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   rain_predict.py
@Path    :   agrothon/server/routers/
@Time    :   2021/05/5
@Author  :   Sreenivas Pakyala
@Version :   1.0.2
@Contact :   sreenivasseenu2904@gmail.com
@Desc    :   RainFall prediction Router
"""
import logging

from fastapi import APIRouter, HTTPException
from fastapi.responses import ORJSONResponse

from ..helpers.rainpredict import rainfall_predict
from ..helpers.response_models import RainPredictionOut, Region

LOGGER = logging.getLogger(__name__)


RainPredictRouter = APIRouter(
    prefix="/rainfall",
    tags=["Rainfall Prediction"],
    responses={404: {"error": "Not found"}},
)


@RainPredictRouter.post(
    "/predict", response_class=ORJSONResponse, response_model=RainPredictionOut
)
async def predict_rainfall(region: Region):
    """
    **Predict Rainfall of the Region based on Previous year's data**
    - **state** : State name
        - ex : Andhra Pradesh
    - **district** : District name
        - ex: East Godavari
    *Returns*
    - Rainfall Predictions between April and December
        - Because in india rainfall mostly happens between these months
    """
    l_state = region.state.lower()
    l_district = region.district.lower()
    u_state = l_state.upper()
    u_district = l_district.upper()
    predictions = rainfall_predict(u_state, u_district)
    response_dict = {
        "state": l_state.title(),
        "district": l_district.title(),
        "units": "mm",
    }
    if predictions is not None:
        response_dict["predictions"] = predictions
        return ORJSONResponse(content=response_dict)
    else:
        raise HTTPException(
            status_code=404, detail={"error": "Check state and district names"}
        )



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   english.py
@Path    :   agrothon/tgbot/translations/
@Time    :   2021/05/24
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   English translations
"""


class Language(object):
    # Main Menu
    MAIN_MENU = "<b>Main Menu</b>"

    # Open Weather
    WEATHER_FETCH = "Fetching Details Please Wait"
    WEATHER = """
<b>📍 Location :</b><code> {}</code>
<b>🌡️ Temperature : </b><code>{} °C</code>
<b>💨 Pressure : </b><code>{} Pa</code>
<b>💧 Humidity : </b><code>{} g.m-3</code>
<b>⛅ Weather : </b><code>{}</code>
"""
    WEATHER_ERR = "<b>Error Occoured</b>"

    # Button Callbacks

    MOISTURE = "Moisture"
    HUMIDITY = "Humidity"
    TEMPERATURE = "Temperature"
    RAIN = "Rainfall Status"
    PUMP_STATUS = "Pump Status"
    COMPLETE_INFO = "Complete info"
    QUIT = "Quit"
    PUMP_OFF = "Switch OFF"
    PUMP_ON = "Switch On"
    BACK = "Back"
    BOT_PRED = "Turn on Bot Prediction"
    REFRESH = "Refresh Status"

    MOISTURE_SENSOR = (
        """<b>💧 Moisture in the Soil (Sensor {}): </b><code> {}%</code>\n"""
    )
    MOISTURE_RESP = """<b>🕒 Last Updated at: </b><code> {}</code>
<b>🕒 Last Read at: </b><code> {}</code>
"""
    HUMID_RESP = """
<b>⛅ Humidity in the Field : </b><code> {}%</code>
<b>🕒 Last Updated at: </b><code> {}</code>
<b>🕒 Last Read at: </b><code> {}</code>
"""
    TEMPE_RESP = """
<b>🌡️ Temparature in the Field : </b><code> {}°C</code>
<b>🕒 Last Updated at: </b><code> {}</code>
<b>🕒 Last Read at: </b><code> {}</code>
"""
    RAIN_YES_RESP = """
<b>It's raining 🌧️ out there</b>

"""
    RAIN_NO_RESP = """
<b>It's not raining 🌞</b>

"""
    COMPLETE_MOISTURE = """<b>💧 Moisture (Sensor {}): </b><code> {}%</code>\n"""
    COMPLETE_RESP = """<b>⛅ Humidity : </b><code> {}%</code>
<b>🌡️ Temperature : </b><code> {}°C</code>
<b>⛏️ Pump Should be (by Sensor {}): </b><code> {}</code>
<b>🕒 Last Updated at: </b><code> {}</code>
<b>🕒 Last Read at: </b><code> {}</code>
"""
    # Pump
    PUMP_SWITCHED_ON = """
<b>The Pump is Running</b>
Switched ON by <code> {}</code>
Last Check : <code> {}</code>
"""

    PUMP_SWITCHED_OFF = """
<b>The Pump is OFF</b>
Switched OFF by <code> {}</code>
Last Check : <code> {}</code>
"""

    BOT_ACTIVATED = """<b>🤖 Bot mode is Activated </b>
Now you can sit back and relax 🛏️ and let 🤖 Bot manage your farm 🚜
"""
    PUMP_BTN_ON = """
<b> ✅ Running Status Changed </b>
Switched ON by<code> User</code>
"""

    PUMP_BTN_OFF = """
<b> ✅ Running Status Changed </b>
Switched OFF by<code> User</code>
"""
    SETTINGS = "⚙️ Settings ⚙️"
    LANG = "🌐 Change Language 🌐"
    SELECT_LANG = "Select Preferred Language"
    LANG_CHANGED = "Changing Language, Please wait..."
    OBJECTS = "Objects"
    DET_NO = "Found"
    ALERT_MESSAGE = """
<b>Intruders has been Detected </b>
<b>Detected at</b> : <code> {}</code>
<b>No of objects detected</b> : <code> {}</code>
<b>No of people detected</b> : <code> {}</code>

<pre>{}</pre>
"""
    MONTHS = [
        "april",
        "may",
        "june",
        "july",
        "august",
        "september",
        "october",
        "november",
        "december",
    ]
    MONTH = "Month"
    RAINFALL = "Rainfall(in {})"
    RAIN_PREDICT = """
<b>Rainfall predictions for this year</b>
<b>State </b>: <code> {}</code>
<b>District </b>: <code> {}</code>

<pre>{}</pre>
"""
    RAIN_PREDICT_ERR = """
Error while
"""
    IMAGE_MESSAGE = """
<b>Objects has been Detected </b>
<b>No of objects detected</b> : <code> {}</code>
<b>No of people detected</b> : <code> {}</code>

<pre>{}</pre>
"""
    ERR_IMAGE_RESPONSE = """
<b>Nothing  Found in the image</b>
"""
    PRED_PUMP_OFF = "Off"
    PRED_PUMP_ON = "On"
    STATS = """
<b>Uptime :</b><code> {}</code>
<b>Disk Space :</b><code> {}</code>
<b>Used :</b><code> {}</code>
<b>Free :</b><code> {}</code>
<b>CPU Usage :</b><code> {}%</code>
<b>RAM :</b><code> {}%</code>
<b>Uploaded :</b><code> {}</code>
<b>Downloaded :</b><code> {}</code>
"""
    DL_TG = "Downloading from Telegram"
    PROC_IMAGE = "Downloaded, Detecting Objects please wait..."
    RESTART = "Restarting, please wait...."
    RESTART_DONE = "Restarted Successfully!"
    RESTART_CALLBACK = "Restart"
    WEATHER_FETCHING = "Fetching Weather, Please Wait"
    HELP_MESSAGE = """
<code>/{}</code> : Weather Status of your City
<code>/{}</code> : Predict rainfall of your region
<code>/{}</code> : Get your field status and manage your pump
<code>/{}</code> : Change settings of your Bot
<code>/{}</code> : Get the server stats
<code>/{}</code> : Check ping
<code>/{}</code> : Get the log of the server
<code>/{}</code> : Restart the server
<code>/{}</code> : To get this message

<code>To detect objects in an image just send the image</code>
"""
    START = """
Hey, I am <code>Agrothon</code>

 - you can monitor your farm
 - Change pump status  to on or off
 - Get weather
 - Detect Objects in an image
 - Predict rainfall for your region
"""
    PING_START = "Starting Ping"
    PING_FINAL = "Measured Ping : {}"
    LANG_SET = "English is successfully set as preferred language"
    PUMP_STATUS_ON = "ON"
    PUMP_STATUS_OFF = "OFF"



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   rainfall.py
@Path    :   agrothon/tgbot/modules/
@Time    :   2021/05/9
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.5
@Contact :   ckvbalusu@gmail.com
@Desc    :   General utils module for Telegram bot
"""

import shutil
import time
from typing import Optional

import psutil

from agrothon import LANG, SERVER_TIME


def get_readable_file_size(size_in_bytes) -> Optional[str]:
    size_units = ["B", "KB", "MB", "GB", "TB", "PB"]
    if size_in_bytes is None:
        return "0B"
    index = 0
    while size_in_bytes >= 1024:
        size_in_bytes /= 1024
        index += 1
    try:
        return f"{round(size_in_bytes, 2)}{size_units[index]}"
    except IndexError:
        return None


def get_readable_time(seconds: int) -> str:
    result = ""
    (days, remainder) = divmod(seconds, 86400)
    days = int(days)
    if days != 0:
        result += f"{days}d"
    (hours, remainder) = divmod(remainder, 3600)
    hours = int(hours)
    if hours != 0:
        result += f"{hours}h"
    (minutes, seconds) = divmod(remainder, 60)
    minutes = int(minutes)
    if minutes != 0:
        result += f"{minutes}m"
    seconds = int(seconds)
    result += f"{seconds}s"
    return result


async def stats(client, message):
    current_time = get_readable_time((time.time() - SERVER_TIME))
    total, used, free = shutil.disk_usage(".")
    total = get_readable_file_size(total)
    used = get_readable_file_size(used)
    free = get_readable_file_size(free)
    cpu_usage = psutil.cpu_percent(interval=0.5)
    memory = psutil.virtual_memory().percent
    sent = get_readable_file_size(psutil.net_io_counters().bytes_sent)
    rec = get_readable_file_size(psutil.net_io_counters().bytes_recv)
    await message.reply_text(
        text=LANG.STATS.format(
            current_time, total, used, free, cpu_usage, memory, sent, rec
        )
    )


async def send_log(client, message):
    await message.reply_document("Agrothon.txt")



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   callbacks.py
@Path    :   agrothon/tgbot/modules/
@Time    :   2021/05/24
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   Callbacks Module for telegram Keyboards
"""
import os
from sys import executable

import dotenv

from agrothon import LANG

from ..helpers.apiserverhelper import *
from ..helpers.keyboards import *


async def callback_sensors(client, message):
    response = await sensor_get_latest()
    if response is not None:
        last_read = response["last_read"]
        up_at = response["updated_at"]
        if message.data == "moisture":
            msg = """"""
            for i in range(response["no_of_sensors"]):
                msg += LANG.MOISTURE_SENSOR.format(str(i + 1), response["moisture"][i])
            msg += LANG.MOISTURE_RESP.format(up_at, last_read)
            await message.message.edit_text(
                text=msg,
                reply_markup=sepkeyboard,
            )
        elif message.data == "humidity":
            await message.message.edit_text(
                text=LANG.HUMID_RESP.format(response["humidity"], up_at, last_read),
                reply_markup=sepkeyboard,
            )
        elif message.data == "temperature":
            await message.message.edit_text(
                text=LANG.TEMPE_RESP.format(response["temperature"], up_at, last_read),
                reply_markup=sepkeyboard,
            )
        elif message.data == "complete":
            pump_ = (
                LANG.PUMP_STATUS_ON
                if response["pump_prediction"]
                else LANG.PUMP_STATUS_OFF
            )
            msg = """"""
            for i in range(response["no_of_sensors"]):
                msg += LANG.COMPLETE_MOISTURE.format(
                    str(i + 1), response["moisture"][i]
                )
            msg += LANG.COMPLETE_RESP.format(
                response["humidity"],
                response["temperature"],
                response["sensor_priority"],
                pump_,
                up_at,
                last_read,
            )
            await message.message.edit_text(
                text=msg,
                reply_markup=sepkeyboard,
            )


async def backcbq(client, message):
    if message.data == "back":
        await message.message.edit_text(text=LANG.MAIN_MENU, reply_markup=fieldkey)
    elif message.data == "exit":
        await message.message.delete()


async def pumpque(client, message):
    if message.data == "pumpstat" or message.data == "refresh":
        response = await pump_get()
        if response is not None:
            if response["status"]:
                await message.message.edit_text(
                    text=LANG.PUMP_SWITCHED_ON.format(
                        response["by"], response["last_read"]
                    ),
                    reply_markup=pumponmenu,
                )
            else:
                await message.message.edit_text(
                    text=LANG.PUMP_SWITCHED_OFF.format(
                        response["by"], response["last_read"]
                    ),
                    reply_markup=pumpoffmenu,
                )
    elif message.data == "pumpon":
        response = await pump_post(True)
        if response is not None:
            await message.message.edit_text(
                text=LANG.PUMP_BTN_ON, reply_markup=pumpoffkey
            )
    elif message.data == "pumpoff":
        response = await pump_post(False)
        if response is not None:
            await message.message.edit_text(
                text=LANG.PUMP_BTN_OFF, reply_markup=pumponkey
            )
    elif message.data == "bot":
        response = await pump_post(True, by="AI Bot")
        if response is not None:
            await message.message.edit_text(
                text=LANG.BOT_ACTIVATED, reply_markup=backkey("pumpstat")
            )


def language_handler(mid, cid):
    with open(".setlangfile", "w") as r_file:
        r_file.truncate(0)
        r_file.write(f"{cid}\n{mid}")
    os.execl(executable, executable, "-m", "agrothon")


async def languages(client, message):
    if message.data == "eng":
        _lang = "english"
        os.environ["DEF_LANG"] = _lang
        dotenv.set_key("agrothon.env", "DEF_LANG", _lang)
        langmsg = await message.message.edit_text(
            text=LANG.LANG_CHANGED, reply_markup=backkey("lang")
        )
        language_handler(langmsg.message_id, langmsg.chat.id)
    elif message.data == "tel":
        _lang = "telugu"
        os.environ["DEF_LANG"] = _lang
        dotenv.set_key("agrothon.env", "DEF_LANG", _lang)
        langmsg = await message.message.edit_text(
            text=LANG.LANG_CHANGED, reply_markup=backkey("lang")
        )
        language_handler(langmsg.message_id, langmsg.chat.id)
    elif message.data == "tam":
        _lang = "tamil"
        os.environ["DEF_LANG"] = _lang
        dotenv.set_key("agrothon.env", "DEF_LANG", _lang)
        langmsg = await message.message.edit_text(
            text=LANG.LANG_CHANGED, reply_markup=backkey("lang")
        )
        language_handler(langmsg.message_id, langmsg.chat.id)
    elif message.data == "hin":
        _lang = "hindi"
        os.environ["DEF_LANG"] = _lang
        dotenv.set_key("agrothon.env", "DEF_LANG", _lang)
        langmsg = await message.message.edit_text(
            text=LANG.LANG_CHANGED, reply_markup=backkey("lang")
        )
        language_handler(langmsg.message_id, langmsg.chat.id)


async def lang_change(client, message):
    await message.message.edit_text(text=LANG.SELECT_LANG, reply_markup=languageskey)


async def restart_callback(client, message):
    restart_msg = await message.message.edit_text(text=LANG.RESTART)
    with open(".restartfile", "w") as r_file:
        r_file.truncate(0)
        r_file.write(f"{restart_msg.chat.id}\n{restart_msg.message_id}")
    os.execl(executable, executable, "-m", "agrothon")



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py
@Path    :   agrothon/tgbot/helpers/
@Time    :   2021/05/3
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.0
@Contact :   ckvbalusu@gmail.com
@Desc    :   Helpers init module
"""



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   apiserverhelper.py
@Path    :   agrothon/tgbot/helpers/
@Time    :   2021/05/24
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   API Server request helper
"""
from json.decoder import JSONDecodeError
from logging import getLogger
from typing import Optional

import aiohttp
import requests

from agrothon import API_BASE_URL, API_KEY

LOGGER = getLogger(__name__)


async def aiohttp_helper(
    url: str, headers: dict, data: Optional[dict] = None, method: Optional[str] = "get"
):
    async with aiohttp.ClientSession() as session:
        if method == "get":
            async with session.get(url=url, headers=headers) as response:
                if response.status == 200:
                    try:
                        data = await response.json()
                        LOGGER.debug(data)
                        return True, data
                    except AttributeError as e:
                        LOGGER.error(e)
                        return False, None
                else:
                    return False, None
        elif method == "post":
            async with session.post(url=url, json=data, headers=headers) as response:
                if response.status == 200:
                    try:
                        data = await response.json()
                        LOGGER.debug(data)
                        return True, data
                    except AttributeError as e:
                        LOGGER.error(e)
                        return False, None
                else:
                    return False, None
        else:
            return False, None


async def pump_get():
    url = f"{API_BASE_URL}pump/?api_key={API_KEY}"
    headers = {"accept": "application/json"}
    status, response = await aiohttp_helper(url=url, headers=headers)
    if status:
        return response
    else:
        return None


async def pump_post(status: bool, by: Optional[str] = "User"):
    url = f"{API_BASE_URL}pump/?api_key={API_KEY}"
    headers = {"accept": "application/json", "Content-Type": "application/json"}
    data = {"status": status, "by": by}
    status, response = await aiohttp_helper(
        url=url, headers=headers, data=data, method="post"
    )
    if status:
        return response
    else:
        return None


async def sensor_get_latest():
    url = f"{API_BASE_URL}field/sensor?api_key={API_KEY}"
    headers = {"accept": "application/json"}
    status, response = await aiohttp_helper(url=url, headers=headers)
    if status:
        return response
    else:
        return None


async def sensor_get_all():
    url = f"{API_BASE_URL}field/sensor/all/?api_key={API_KEY}"
    headers = {"accept": "application/json"}
    status, response = await aiohttp_helper(url=url, headers=headers)
    if status:
        return response
    else:
        return None


async def open_weather(city: str):
    url = f"{API_BASE_URL}weather/{city}?api_key={API_KEY}"
    headers = {"accept": "application/json"}
    status, response = await aiohttp_helper(url=url, headers=headers)
    if status:
        return response
    else:
        return None


def get_image_uuids():
    url = f"{API_BASE_URL}intruder/images/uuids?api_key={API_KEY}"
    headers = {"accept": "application/json"}
    try:
        response = requests.get(url, headers=headers)
        if response.status_code == 200:
            resp = response.json()
            LOGGER.info(f"Getting UUIDS, Response : {str(resp)}")
            return resp
        else:
            return None
    except Exception as e:
        LOGGER.error(e)
        return None
    except JSONDecodeError:
        LOGGER.error(f"Json Decode Error")
        return None


def get_image_url(uuid: str):
    url = f"{API_BASE_URL}intruder/detect/image/{uuid}?api_key={API_KEY}"
    return url


async def get_rainfall_prediction(state: str, district: str):
    url = f"{API_BASE_URL}rainfall/predict?api_key={API_KEY}"
    headers = {"accept": "application/json"}
    data = {"state": state, "district": district}
    status, response = await aiohttp_helper(
        url=url, headers=headers, data=data, method="post"
    )
    if status:
        return response
    else:
        return None


async def get_instant_image_url(uuid: str):
    url = f"{API_BASE_URL}intruder/detect/instant/{uuid}?api_key={API_KEY}"
    return url


async def upload_file_to_api(path):
    url = f"{API_BASE_URL}intruder/detect/instant?api_key={API_KEY}"
    with open(path, "rb") as file:
        files = {"image": file}
        async with aiohttp.ClientSession() as session:
            async with session.post(url=url, data=files) as response:
                if response.status == 200:
                    try:
                        data = await response.json()
                        image_url = await get_instant_image_url(data["uuid"])
                        del data["uuid"]
                        data["image_url"] = image_url
                        return True, data
                    except AttributeError:
                        return False, None
                else:
                    return False, None



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   photo_handler.py
@Path    :   agrothon/tgbot/modules/
@Time    :   2021/05/9
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.5
@Contact :   ckvbalusu@gmail.com
@Desc    :   Pyrogram Photo Filter to detect objects
"""
import os
import uuid

from prettytable import PrettyTable

from agrothon import LANG

from ..helpers.apiserverhelper import upload_file_to_api


async def photo_detect(client, message):
    temp_uuid = str(uuid.uuid4())
    path = f"tmp/{temp_uuid}.jpg"
    init_msg = await message.reply_text(text=LANG.DL_TG)
    downloaded_file = await client.download_media(message=message, file_name=path)
    if os.path.exists(downloaded_file):
        proc_message = await init_msg.edit_text(text=LANG.PROC_IMAGE)
        status, image = await upload_file_to_api(downloaded_file)
        if status:
            pt = PrettyTable([LANG.OBJECTS, LANG.DET_NO])
            pt.align[LANG.OBJECTS] = "l"
            pt.align[LANG.DET_NO] = "c"
            pt.padding_width = 0
            pt.format = True
            image_url = image["image_url"]
            detections = image["detections"]
            hums = image["humans"]
            tot_dets = image["no_of_detections"]
            for obj in detections:
                pt.add_row([obj["type"], obj["count"]])
            await proc_message.delete()
            await message.reply_photo(
                photo=image_url,
                quote=True,
                caption=LANG.IMAGE_MESSAGE.format(tot_dets, hums, pt),
                parse_mode="html",
            )
        else:
            await proc_message.edit_text(text=LANG.ERR_IMAGE_RESPONSE)
        os.remove(downloaded_file)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   settings.py
@Path    :   agrothon/tgbot/modules/
@Time    :   2021/05/9
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.5
@Contact :   ckvbalusu@gmail.com
@Desc    :   Settings Command handler
"""
import os
import time
from sys import executable

from agrothon import (
    FIELD_COMMAND,
    HELP_COMMAND,
    LANG,
    LOG_COMMAND,
    PING_COMMAND,
    RAIN_COMMAND,
    RESTART_COMMAND,
    SETTINGS_COMMAND,
    STATS_COMMAND,
    WEATHER_COMMAND,
)

from ..helpers.keyboards import settingskey


async def settings(client, message):
    await message.reply_text(text=LANG.SETTINGS, reply_markup=settingskey)


async def restart(client, message):
    restart_msg = await message.reply_text(text=LANG.RESTART)
    with open(".restartfile", "w") as r_file:
        r_file.truncate(0)
        r_file.write(f"{restart_msg.chat.id}\n{restart_msg.message_id}")
    os.execl(executable, executable, "-m", "agrothon")


async def start(client, message):
    await message.reply_text(text=LANG.START)


async def help_command(client, message):
    await message.reply_text(
        text=LANG.HELP_MESSAGE.format(
            WEATHER_COMMAND,
            RAIN_COMMAND,
            FIELD_COMMAND,
            SETTINGS_COMMAND,
            STATS_COMMAND,
            PING_COMMAND,
            LOG_COMMAND,
            RESTART_COMMAND,
            HELP_COMMAND,
        )
    )


async def ping_command(client, message):
    start_time = int(round(time.time() * 1000))
    ping_start = await message.reply_text(text=LANG.PING_START)
    end_time = int(round(time.time() * 1000))
    await ping_start.edit_text(text=LANG.PING_FINAL.format(f"{end_time-start_time} ms"))



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py
@Path    :   agrothon/server/helpers/
@Time    :   2021/05/3
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.2
@Contact :   ckvbalusu@gmail.com
@Desc    :   helpers init module
"""
import logging
import os

logging.getLogger("matplotlib").setLevel(logging.WARNING)
logging.getLogger("tensorflow").setLevel(logging.ERROR)
logging.getLogger("multipart").setLevel(logging.ERROR)
os.environ["KMP_AFFINITY"] = "noverbose"
os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   rainpredict.py
@Path    :   agrothon/server/helpers/
@Time    :   2021/05/5
@Author  :   Sreenivas Pakyala
@Version :   1.0.3
@Contact :   sreenivasseenu2904@gmail.com
@Desc    :   Rain Prediction Deep learning Module
"""

import logging
from typing import Dict, Optional

import numpy as np
import pandas as pd
import tensorflow as tf
from keras.models import model_from_json

tf.get_logger().setLevel("ERROR")
tf.autograph.set_verbosity(3)
LOGGER = logging.getLogger(__name__)


with open("data/models/rainfall/rainfall_model.json") as json_file:
    loaded_model = model_from_json(json_file.read())
csv_data = pd.read_csv("data/models/rainfall/india_rainfall.csv")
loaded_model.load_weights("data/models/rainfall/rainfall_model.h5")
_MONTHS = [
    "DISTRICT",
    "JAN",
    "FEB",
    "MAR",
    "APR",
    "MAY",
    "JUN",
    "JUL",
    "AUG",
    "SEP",
    "OCT",
    "NOV",
    "DEC",
]
MONTHS = [
    "JAN",
    "FEB",
    "MAR",
    "APR",
    "MAY",
    "JUN",
    "JUL",
    "AUG",
    "SEP",
    "OCT",
    "NOV",
    "DEC",
]
MONTHS_ = [
    "april",
    "may",
    "june",
    "july",
    "august",
    "september",
    "october",
    "november",
    "december",
]


def rainfall_predict(state: str, district: str) -> Optional[Dict]:
    try:
        LOGGER.info(f"Getting predictions for {district} in {state}")
        filter_state = csv_data[_MONTHS].loc[csv_data["STATE/UT"] == state]
        filter_district = np.asarray(
            filter_state[MONTHS].loc[filter_state["DISTRICT"] == district]
        )
        x_year = None
        for i in range(filter_district.shape[1] - 3):
            if x_year is None:
                x_year = filter_district[:, i : i + 3]
            else:
                x_year = np.concatenate((x_year, filter_district[:, i : i + 3]), axis=0)
        y_year_pred = loaded_model.predict(np.expand_dims(x_year, axis=2))
        predictions_dict = {}
        if y_year_pred is not None:
            for i in range(len(y_year_pred)):
                predictions_dict[MONTHS_[i]] = round(float(y_year_pred[i]), 2)
            LOGGER.debug(f"Predicted Rainfall Successfully : {str(predictions_dict)}")
            return predictions_dict
        else:
            LOGGER.error(f"Prediction of rainfall returned None")
            return None
    except Exception as e:
        LOGGER.error(f"Error in predicting Rainfall : {e}")
        return None



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   weather.py
@Path    :   agrothon/tgbot/modules/
@Time    :   2021/05/9
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.5
@Contact :   ckvbalusu@gmail.com
@Desc    :   Weather Command handler
"""

from agrothon import DEF_CITY, LANG

from ..helpers.apiserverhelper import open_weather


async def weather(client, message):
    init = await message.reply_text(LANG.WEATHER_FETCHING)
    response = await open_weather(DEF_CITY)
    if response is not None:
        msg = LANG.WEATHER.format(
            response["city"],
            response["temperature"],
            response["pressure"],
            response["humidity"],
            response["weather"],
        )
        await init.edit_text(msg)
    else:
        await init.edit_text(LANG.WEATHER_ERR.format())



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   tamil.py
@Path    :   agrothon/tgbot/translations/
@Time    :   2021/05/24
@Author  :   Sreenivas Pakyala
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   Tamil translations
"""


class Language(object):
    # Main Menu
    MAIN_MENU = "<b>முதன்மை பட்டியல்</b>"

    # Open Weather
    WEATHER_FETCH = "விவரங்களைப் பெறுதல் தயவுசெய்து காத்திருங்கள்"
    WEATHER = """
<b>📍 இடம் :</b><code> {}</code>
<b>🌡️ வெப்ப நிலை : </b><code>{} °C</code>
<b>💨 அழுத்தம் : </b><code>{} Pa</code>
<b>💧 ஈரப்பதம் : </b><code>{} g.m-3</code>
<b>⛅ வானிலை : </b><code>{}</code>
"""
    WEATHER_ERR = "<b>பிழை ஏற்பட்டது, பதில் : <code>{}</code> </b>"

    # Button Callbacks

    MOISTURE = "ஈரம்"
    HUMIDITY = "ஈரப்பதம்"
    TEMPERATURE = "வெப்ப நிலை"
    RAIN = "மழைக்கால நிலை"
    PUMP_STATUS = "பம்ப் நிலை"
    COMPLETE_INFO = "முழுமையான தகவல்"
    QUIT = "விட்டுவிட"
    PUMP_OFF = "அனைத்து விடு"
    PUMP_ON = "மாறவும்"
    BACK = "மீண்டும்"
    BOT_PRED = "பாட் ப்ரிடிக்டனை இயக்கவும்"
    REFRESH = "நிலையை புதுப்பிக்கவும்"

    MOISTURE_SENSOR = """<b>💧 மண்ணில் ஈரப்பதம் (சென்சார் {}): </b><code> {}%</code>\n"""
    MOISTURE_RESP = """<b>🕒 கடைசியாக புதுப்பிக்கப்பட்டது: </b><code> {}</code>
<b>🕒 கடைசியாக படித்தது: </b><code> {}</code>
"""
    HUMID_RESP = """
<b>⛅ புலத்தில் ஈரப்பதம் : </b><code> {}%</code>
<b>🕒 கடைசியாக புதுப்பிக்கப்பட்டது: </b><code> {}</code>
<b>🕒 கடைசியாக படித்தது: </b><code> {}</code>
"""
    TEMPE_RESP = """
<b>🌡️ புலத்தில் தற்காலிகம் : </b><code> {}°C</code>
<b>🕒 கடைசியாக புதுப்பிக்கப்பட்டது: </b><code> {}</code>
<b>🕒 கடைசியாக படித்தத: </b><code> {}</code>
"""
    RAIN_YES_RESP = """
<b>பண்ணையில் மழை பெய்கிறது 🌧️ </b>

"""
    RAIN_NO_RESP = """
<b>பண்ணையில் மழை பெய்யவில்லை 🌞</b>

"""
    COMPLETE_MOISTURE = """<b>💧 ஈரம் (சென்சார் {}): </b><code> {}%</code>\n"""
    COMPLETE_RESP = """<b>⛅ ஈரப்பதம் : </b><code> {}%</code>
<b>🌡️ வெப்ப நில : </b><code> {}°C</code>
<b>⛏️ பம்ப் இருக்க வேண்டும் (சென்சார் {}): </b><code> {}</code>
<b>🕒 கடைசியாக புதுப்பிக்கப்பட்டது: </b><code> {}</code>
<b>🕒 கடைசியாக படித்தது: </b><code> {}</code>
"""
    # Pump
    PUMP_SWITCHED_ON = """
<b>பம்ப் இயங்குகிறது</b>
இயக்கப்பட்டது <code> {}</code>
கடைசி சோதனை : <code> {}</code>
"""

    PUMP_SWITCHED_OFF = """
<b>பம்ப் முடக்கப்பட்டுள்ளது</b>
ஆல் மாற்றப்பட்டது <code> {}</code>
கடைசி சோதனை : <code> {}</code>
"""

    BOT_ACTIVATED = """<b>🤖 போட் பயன்முறை செயல்படுத்தப்பட்டது</b>
இப்போது நீங்கள் உட்கார்ந்து ஓய்வெடுக்கலாம் மற்றும் உங்கள் 🤖 பண்ணையை நிர்வகிக்க போட் அனுமதிக்கலாம் 🚜
"""
    PUMP_BTN_ON = """
<b> ✅ இயங்கும் நிலை மாற்றப்பட்டது </b>
இயக்கப்பட்டது<code> User</code>
"""

    PUMP_BTN_OFF = """
<b> ✅ இயங்கும் நிலை மாற்றப்பட்டது </b>
ஆல் மாற்றப்பட்டது<code> User</code>
"""

    SETTINGS = "⚙️ அமைப்புகள் ⚙️"
    LANG = "🌐 மொழியை மாற்றுங்கள் 🌐"
    SELECT_LANG = "விருப்பமான மொழியைத் தேர்ந்தெடுக்கவும்"
    LANG_CHANGED = "மொழி ஆங்கிலத்திற்கு மாற்றப்பட்டுள்ளது"
    OBJECTS = "பொருள்கள்"
    DET_NO = "கண்டறியப்பட்டது"
    ALERT_MESSAGE = """
<b>ஊடுருவும் நபர்கள் கண்டறியப்பட்டுள்ளனர் </b>
<b>இல் கண்டறியப்பட்டது</b> : <code> {}</code>
<b>பொருள்கள் எதுவும் கண்டறியப்படவில்லை</b> : <code> {}</code>
<b>மக்கள் யாரும் கண்டறியப்படவில்லை</b> : <code> {}</code>

<pre>{}</pre>
"""
    MONTHS = [
        "ஏப்ரல்",
        "மே",
        "ஜூன்",
        "ஜூலை",
        "ஆகஸ்ட்",
        "செப்டம்பர்",
        "அக்டோபர்",
        "நவம்பர்",
        "டிசம்பர்",
    ]
    MONTH = "மாதம்"
    RAINFALL = "மழைப்பொழிவு(இல் {})"
    RAIN_PREDICT = """
<b>இந்த ஆண்டிற்கான மழைப்பொழிவு கணிப்புகள்</b>
<b>நிலை</b>: <code> {}</code>
<b>மாவட்டம் </b>: <code> {}</code>

<pre>{}</pre>
"""
    RAIN_PREDICT_ERR = """
கணிக்கும் போது பிழை
"""
    IMAGE_MESSAGE = """
<b>பொருள்கள் கண்டறியப்பட்டுள்ளன </b>
<b>பொருள்கள் எதுவும் கண்டறியப்படவில்லை</b> : <code> {}</code>
<b>மக்கள் யாரும் கண்டறியப்படவில்லை</b> : <code> {}</code>

<pre>{}</pre>
"""
    ERR_IMAGE_RESPONSE = """
<b>படத்தில் எதுவும் காணப்படவில்லை</b>
"""
    PRED_PUMP_OFF = "மஆஃப்ு"
    PRED_PUMP_ON = "ஆன்"
    STATS = """
<b>முடிந்தநேரம் :</b><code> {}</code>
<b>வட்டு அளவு :</b><code> {}</code>
<b>பயன்படுத்தப்பட்டது :</b><code> {}</code>
<b>இலவசம் :</b><code> {}</code>
<b>CPU பயன்பாடு :</b><code> {}%</code>
<b>RAM பயன்பாடு :</b><code> {}%</code>
<b>பதிவேற்றப்பட்டது :</b><code> {}</code>
<b>பதிவிறக்கம் செய்யப்பட்டது :</b><code> {}</code>
"""
    DL_TG = "டெலிகிராமிலிருந்து பதிவிறக்குகிறது"
    PROC_IMAGE = (
        "Dசொந்தமாக ஏற்றப்பட்டது, பொருள்களைக் கண்டறிதல் தயவுசெய்து காத்திருங்கள் ..."
    )
    RESTART = "மறுதொடக்கம், தயவுசெய்து காத்திருங்கள் ...."
    RESTART_DONE = "மீண்டும் தொடங்கப்பட்டது வெற்றிகரமாக!"
    RESTART_CALLBACK = "மறுதொடக்கம்"
    WEATHER_FETCHING = "வானிலை பெறுகிறது, தயவுசெய்து காத்திருங்கள்"
    HELP_MESSAGE = """
<code>/{}</code> : உங்கள் நகரத்தின் வானிலை நிலை
<code>/{}</code> : உங்கள் பிராந்தியத்தின் மழையை கணிக்கவும்
<code>/{}</code> : உங்கள் புல நிலையைப் பெற்று உங்கள் பம்பை நிர்வகிக்கவும்
<code>/{}</code> : உங்கள் போட் அமைப்புகளை மாற்றவும்
<code>/{}</code> : சேவையக புள்ளிவிவரங்களைப் பெறுங்கள்
<code>/{}</code> : பிங் சரிபார்க்கவும்
<code>/{}</code> : சேவையகத்தின் பதிவைப் பெறுங்கள்
<code>/{}</code> : சேவையகத்தை மறுதொடக்கம் செய்யுங்கள்
<code>/{}</code> : இந்த செய்தியைப் பெற

<code>ஒரு படத்தில் உள்ள பொருட்களைக் கண்டறிய படத்தை அனுப்பவும்</code>
"""
    START = """
ஏய், நான் <code>அக்ரோத்தான்</code>

 - உங்கள் பண்ணையை கண்காணிக்கலாம்
 - பம்ப் நிலையை ஆன் அல்லது ஆஃப் என மாற்றவும்
 - வானிலை கிடைக்கும்
 - ஒரு படத்தில் பொருள்களைக் கண்டறியவும்
 - உங்கள் பிராந்தியத்திற்கு மழையை கணிக்கவும்
"""
    PING_START = "பிங் தொடங்குகிறது"
    PING_FINAL = "அளவிடப்பட்ட பிங் : {}"
    LANG_SET = "விருப்பமான மொழி தொகுப்பு வெற்றிகரமாக"
    PUMP_STATUS_ON = "மீது"
    PUMP_STATUS_OFF = "ஆஃப்"



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File        :   yolov3_helper.py
@Path        :   agrothon/server/helpers/
@Time        :   2021/05/28
@Author      :   github.com/zzh8829
@Modifed by  :   Chandra Kiran Viswanath Balusu
@Version     :   1.3.1
@Contact     :   ckvbalusu@gmail.com
@Desc        :   Yolov3 Python3 Wrapper using tf2
"""

import logging
import os
from pathlib import Path
from typing import List, Optional, Tuple

import cv2
import numpy as np
import tensorflow as tf
from PIL import Image, ImageDraw, ImageFont
from seaborn import color_palette
from tensorflow.keras import Model
from tensorflow.keras.layers import (
    Add,
    BatchNormalization,
    Concatenate,
    Conv2D,
    Input,
    Lambda,
    LeakyReLU,
    UpSampling2D,
    ZeroPadding2D,
)
from tensorflow.keras.regularizers import l2

tf.get_logger().setLevel("ERROR")
tf.autograph.set_verbosity(3)

LOGGER = logging.getLogger(__name__)
yolo_max_boxes = 100  # maximum number of boxes per image
yolo_iou_threshold = 0.5  # iou threshold
yolo_score_threshold = 0.5  # score threshold
yolo_anchors = (
    np.array(
        [
            (10, 13),
            (16, 30),
            (33, 23),
            (30, 61),
            (62, 45),
            (59, 119),
            (116, 90),
            (156, 198),
            (373, 326),
        ],
        np.float32,
    )
    / 416
)
yolo_anchor_masks = np.array([[6, 7, 8], [3, 4, 5], [0, 1, 2]])

YOLOV3_LAYER_LIST = [
    "yolo_darknet",
    "yolo_conv_0",
    "yolo_output_0",
    "yolo_conv_1",
    "yolo_output_1",
    "yolo_conv_2",
    "yolo_output_2",
]

""" --------------------------------Models------------------------------ """


def DarknetConv(x, filters, size, strides=1, batch_norm=True):
    if strides == 1:
        padding = "same"
    else:
        x = ZeroPadding2D(((1, 0), (1, 0)))(x)
        padding = "valid"
    x = Conv2D(
        filters=filters,
        kernel_size=size,
        strides=strides,
        padding=padding,
        use_bias=not batch_norm,
        kernel_regularizer=l2(0.0005),
    )(x)
    if batch_norm:
        x = BatchNormalization()(x)
        x = LeakyReLU(alpha=0.1)(x)
    return x


def DarknetResidual(x, filters):
    prev = x
    x = DarknetConv(x, filters // 2, 1)
    x = DarknetConv(x, filters, 3)
    x = Add()([prev, x])
    return x


def DarknetBlock(x, filters, blocks):
    x = DarknetConv(x, filters, 3, strides=2)
    for _ in range(blocks):
        x = DarknetResidual(x, filters)
    return x


def Darknet(name=None):
    x = inputs = Input([None, None, 3])
    x = DarknetConv(x, 32, 3)
    x = DarknetBlock(x, 64, 1)
    x = DarknetBlock(x, 128, 2)
    x = x_36 = DarknetBlock(x, 256, 8)
    x = x_61 = DarknetBlock(x, 512, 8)
    x = DarknetBlock(x, 1024, 4)
    return tf.keras.Model(inputs, (x_36, x_61, x), name=name)


def YoloConv(filters, name=None):
    def yolo_conv(x_in):
        if isinstance(x_in, tuple):
            inputs = Input(x_in[0].shape[1:]), Input(x_in[1].shape[1:])
            x, x_skip = inputs
            x = DarknetConv(x, filters, 1)
            x = UpSampling2D(2)(x)
            x = Concatenate()([x, x_skip])
        else:
            x = inputs = Input(x_in.shape[1:])

        x = DarknetConv(x, filters, 1)
        x = DarknetConv(x, filters * 2, 3)
        x = DarknetConv(x, filters, 1)
        x = DarknetConv(x, filters * 2, 3)
        x = DarknetConv(x, filters, 1)
        return Model(inputs, x, name=name)(x_in)

    return yolo_conv


def YoloOutput(filters, anchors, classes, name=None):
    def yolo_output(x_in):
        x = inputs = Input(x_in.shape[1:])
        x = DarknetConv(x, filters * 2, 3)
        x = DarknetConv(x, anchors * (classes + 5), 1, batch_norm=False)
        x = Lambda(
            lambda x: tf.reshape(
                x, (-1, tf.shape(x)[1], tf.shape(x)[2], anchors, classes + 5)
            )
        )(x)
        return tf.keras.Model(inputs, x, name=name)(x_in)

    return yolo_output


def _meshgrid(n_a, n_b):
    return [
        tf.reshape(tf.tile(tf.range(n_a), [n_b]), (n_b, n_a)),
        tf.reshape(tf.repeat(tf.range(n_b), n_a), (n_b, n_a)),
    ]


def yolo_boxes(pred, anchors, classes):
    grid_size = tf.shape(pred)[1:3]
    box_xy, box_wh, objectness, class_probs = tf.split(
        pred, (2, 2, 1, classes), axis=-1
    )

    box_xy = tf.sigmoid(box_xy)
    objectness = tf.sigmoid(objectness)
    class_probs = tf.sigmoid(class_probs)
    pred_box = tf.concat((box_xy, box_wh), axis=-1)

    grid = _meshgrid(grid_size[1], grid_size[0])
    grid = tf.expand_dims(tf.stack(grid, axis=-1), axis=2)

    box_xy = (box_xy + tf.cast(grid, tf.float32)) / tf.cast(grid_size, tf.float32)
    box_wh = tf.exp(box_wh) * anchors

    box_x1y1 = box_xy - box_wh / 2
    box_x2y2 = box_xy + box_wh / 2
    bbox = tf.concat([box_x1y1, box_x2y2], axis=-1)

    return bbox, objectness, class_probs, pred_box


def yolo_nms(outputs, anchors, masks, classes):
    b, c, t = [], [], []

    for o in outputs:
        b.append(tf.reshape(o[0], (tf.shape(o[0])[0], -1, tf.shape(o[0])[-1])))
        c.append(tf.reshape(o[1], (tf.shape(o[1])[0], -1, tf.shape(o[1])[-1])))
        t.append(tf.reshape(o[2], (tf.shape(o[2])[0], -1, tf.shape(o[2])[-1])))

    bbox = tf.concat(b, axis=1)
    confidence = tf.concat(c, axis=1)
    class_probs = tf.concat(t, axis=1)

    scores = confidence * class_probs

    dscores = tf.squeeze(scores, axis=0)
    scores = tf.reduce_max(dscores, [1])
    bbox = tf.reshape(bbox, (-1, 4))
    classes = tf.argmax(dscores, 1)
    selected_indices, selected_scores = tf.image.non_max_suppression_with_scores(
        boxes=bbox,
        scores=scores,
        max_output_size=yolo_max_boxes,
        iou_threshold=yolo_iou_threshold,
        score_threshold=yolo_score_threshold,
        soft_nms_sigma=0.5,
    )

    num_valid_nms_boxes = tf.shape(selected_indices)[0]

    selected_indices = tf.concat(
        [selected_indices, tf.zeros(yolo_max_boxes - num_valid_nms_boxes, tf.int32)], 0
    )
    selected_scores = tf.concat(
        [selected_scores, tf.zeros(yolo_max_boxes - num_valid_nms_boxes, tf.float32)],
        -1,
    )

    boxes = tf.gather(bbox, selected_indices)
    boxes = tf.expand_dims(boxes, axis=0)
    scores = selected_scores
    scores = tf.expand_dims(scores, axis=0)
    classes = tf.gather(classes, selected_indices)
    classes = tf.expand_dims(classes, axis=0)
    valid_detections = num_valid_nms_boxes
    valid_detections = tf.expand_dims(valid_detections, axis=0)

    return boxes, scores, classes, valid_detections


def YoloV3(
    size=None,
    channels=3,
    anchors=yolo_anchors,
    masks=yolo_anchor_masks,
    classes=80,
):
    x = inputs = Input([size, size, channels], name="input")

    x_36, x_61, x = Darknet(name="yolo_darknet")(x)

    x = YoloConv(512, name="yolo_conv_0")(x)
    output_0 = YoloOutput(512, len(masks[0]), classes, name="yolo_output_0")(x)

    x = YoloConv(256, name="yolo_conv_1")((x, x_61))
    output_1 = YoloOutput(256, len(masks[1]), classes, name="yolo_output_1")(x)

    x = YoloConv(128, name="yolo_conv_2")((x, x_36))
    output_2 = YoloOutput(128, len(masks[2]), classes, name="yolo_output_2")(x)

    boxes_0 = Lambda(
        lambda x: yolo_boxes(x, anchors[masks[0]], classes), name="yolo_boxes_0"
    )(output_0)
    boxes_1 = Lambda(
        lambda x: yolo_boxes(x, anchors[masks[1]], classes), name="yolo_boxes_1"
    )(output_1)
    boxes_2 = Lambda(
        lambda x: yolo_boxes(x, anchors[masks[2]], classes), name="yolo_boxes_2"
    )(output_2)

    outputs = Lambda(lambda x: yolo_nms(x, anchors, masks, classes), name="yolo_nms")(
        (boxes_0[:3], boxes_1[:3], boxes_2[:3])
    )

    return Model(inputs, outputs, name="yolov3")


""" --------------------------------Utils------------------------------ """


def transform_images(x_train, size):
    x_train = tf.image.resize(x_train, (size, size))
    x_train = x_train / 255
    return x_train


def load_darknet_weights(model, weights_file):
    wf = open(weights_file, "rb")
    major, minor, revision, seen, _ = np.fromfile(wf, dtype=np.int32, count=5)

    layers = YOLOV3_LAYER_LIST

    for layer_name in layers:
        sub_model = model.get_layer(layer_name)
        for i, layer in enumerate(sub_model.layers):
            if not layer.name.startswith("conv2d"):
                continue
            batch_norm = None
            if i + 1 < len(sub_model.layers) and sub_model.layers[
                i + 1
            ].name.startswith("batch_norm"):
                batch_norm = sub_model.layers[i + 1]

            filters = layer.filters
            size = layer.kernel_size[0]
            in_dim = layer.get_input_shape_at(0)[-1]

            if batch_norm is None:
                conv_bias = np.fromfile(wf, dtype=np.float32, count=filters)
            else:
                bn_weights = np.fromfile(wf, dtype=np.float32, count=4 * filters)
                bn_weights = bn_weights.reshape((4, filters))[[1, 0, 2, 3]]

            conv_shape = (filters, in_dim, size, size)
            conv_weights = np.fromfile(
                wf, dtype=np.float32, count=np.product(conv_shape)
            )
            conv_weights = conv_weights.reshape(conv_shape).transpose([2, 3, 1, 0])

            if batch_norm is None:
                layer.set_weights([conv_weights, conv_bias])
            else:
                layer.set_weights([conv_weights])
                batch_norm.set_weights(bn_weights)

    assert len(wf.read()) == 0, "failed to read all data"
    wf.close()


def draw_outputs(img, outputs, class_names):
    colors = (np.array(color_palette("hls", 80)) * 255).astype(np.uint8)
    boxes, objectness, classes, nums = outputs
    wh = np.flip(img.shape[0:2])
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    img = Image.fromarray(img)
    draw = ImageDraw.Draw(img)
    font = ImageFont.truetype(
        font="./data/fonts/futur.ttf", size=(img.size[0] + img.size[1]) // 100
    )
    for i in range(nums):
        color = colors[int(classes[i])]
        x1y1 = (np.array(boxes[i][0:2]) * wh).astype(np.int32)
        x2y2 = (np.array(boxes[i][2:4]) * wh).astype(np.int32)
        thickness = (img.size[0] + img.size[1]) // 200
        x0, y0 = x1y1[0], x1y1[1]
        for t in np.linspace(0, 1, thickness):
            x1y1[0], x1y1[1] = x1y1[0] - t, x1y1[1] - t
            x2y2[0], x2y2[1] = x2y2[0] - t, x2y2[1] - t
            draw.rectangle([x1y1[0], x1y1[1], x2y2[0], x2y2[1]], outline=tuple(color))
        confidence = "{:.2f}%".format(objectness[i] * 100)
        text = "{} {}".format(class_names[int(classes[i])], confidence)
        text_size = draw.textsize(text, font=font)
        draw.rectangle(
            [x0, y0 - text_size[1], x0 + text_size[0], y0], fill=tuple(color)
        )
        draw.text((x0, y0 - text_size[1]), text, fill="black", font=font)
    rgb_img = img.convert("RGB")
    img_np = np.asarray(rgb_img)
    img = cv2.cvtColor(img_np, cv2.COLOR_BGR2RGB)

    return img


""" --------------------------------Methods to Detect image goes here------------------------------ """

_yolo = YoloV3(classes=80)
gen_exist = False
try:
    paths = []
    for path in Path("data/models/yolo/weights/").glob("yolov3.tf*"):
        paths.append(path)
    if len(paths) == 2:
        gen_exist = True
        raise FileExistsError
except FileExistsError:
    _yolo.load_weights("data/models/yolo/weights/yolov3.tf").expect_partial()
    LOGGER.info("Loaded Generated Weights")

try:
    if not gen_exist:
        if os.path.exists("data/models/yolo/yolov3.weights"):
            load_darknet_weights(_yolo, "data/models/yolo/yolov3.weights")
            LOGGER.info("Loaded Yolov3 Weights")
            _yolo.save_weights("data/models/yolo/weights/yolov3.tf")
            LOGGER.info("Saved Yolov3 Weights")
            _yolo.load_weights("data/models/yolo/weights/yolov3.tf").expect_partial()
            LOGGER.info("Loaded Generated Weights")
        else:
            raise FileNotFoundError
except FileNotFoundError:
    LOGGER.error("Neither Generated weights or yolov3.weights not found")
    exit(1)
try:
    if os.path.exists("data/models/yolo/coco.names"):
        _class_names = [
            c.strip() for c in open("data/models/yolo/coco.names").readlines()
        ]
        LOGGER.info("Loaded classes from coco.names")
    else:
        raise FileNotFoundError
except FileNotFoundError:
    LOGGER.error(f"Coco.names not found :(")
    exit(1)


# INCLUDED_CLASSES = ["person", "bird", "cat", "dog", "horse", "sheep", "cow", "elephant", "bear", "zebra", "giraffe"]
INC_CLASS_NUMBERS = [0, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23]


def yolo_detect(
    _image: bytes,
    filter: bool = True,
) -> Tuple[
    Optional[bool],
    Optional[int],
    Optional[List],
    Optional[int],
    Optional[bool],
    Optional[bytes],
]:
    try:
        LOGGER.info("Started detection of Objects in the image")
        img_raw = tf.image.decode_image(_image, channels=3)
        _img_ = tf.expand_dims(img_raw, 0)
        img__ = transform_images(_img_, 416)
        _boxes, _scores, _classes, _nums = _yolo(img__)
        filtered_boxes, filtered_scores, filtered_classes, new_nums = [], [], [], 0
        if filter:
            for i in range(_nums[0]):
                if int(_classes[0][i]) in INC_CLASS_NUMBERS:
                    filtered_classes.append(int(_classes[0][i]))
                    filtered_boxes.append(_boxes[0][i])
                    filtered_scores.append(_scores[0][i])
                    new_nums += 1
        else:
            filtered_classes = _classes[0]
            filtered_boxes = _boxes[0]
            filtered_scores = _scores[0]
            new_nums = _nums[0]

        if new_nums > 0:
            detections_list = []
            humans = 0
            only_humans = False
            _objects = {}
            _confidences = {}
            for i in range(new_nums):
                if str(_class_names[filtered_classes[i]]) in _objects:

                    _objects[str(_class_names[filtered_classes[i]])] += 1
                    _confidences[str(_class_names[filtered_classes[i]])].append(
                        round(float(np.array(filtered_scores[i])) * 100, 2)
                    )
                else:
                    _objects[str(_class_names[filtered_classes[i]])] = 1
                    _confidences[str(_class_names[filtered_classes[i]])] = [
                        round(float(np.array(filtered_scores[i])) * 100, 2)
                    ]

                if (_class_names[int(filtered_classes[i])]) == "person":
                    humans += 1
            for i in _objects:
                detection_dict = {
                    "type": i,
                    "confidences": _confidences[i],
                    "count": _objects[i],
                }
                detections_list.append(detection_dict)

            if _nums[0] == humans:
                only_humans = True
            cv2_img = cv2.cvtColor(img_raw.numpy(), cv2.COLOR_RGB2BGR)
            cv2_img = draw_outputs(
                cv2_img,
                (filtered_boxes, filtered_scores, filtered_classes, new_nums),
                _class_names,
            )
            is_success, cv2_img = cv2.imencode(".jpg", cv2_img)
            LOGGER.debug(f"Finished Detecting, Objects found : {str(detections_list)}")
            return True, int(new_nums), detections_list, humans, only_humans, cv2_img
        else:
            LOGGER.debug(f"Finished Detecting, Nothing Found in the image")
            return False, None, None, None, None, None
    except Exception as e:
        LOGGER.error(f"Error occurred while predicting : {str(e)}")
        return None, None, None, None, None, None



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   openwapi.py
@Path    :   agrothon/server/helpers/
@Time    :   2021/05/5
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.2
@Contact :   ckvbalusu@gmail.com
@Desc    :   Open Weather API Request Module
"""
from typing import Dict, Optional, Tuple

import aiohttp

from agrothon import LOGGER, OPEN_WEATHER_API


async def open_weather_helper_city(city: str) -> Tuple[bool, Optional[Dict]]:
    async with aiohttp.ClientSession() as session:
        api_url = f"https://api.openweathermap.org/data/2.5/weather?q={city}&appid={OPEN_WEATHER_API}&units=metric"
        LOGGER.debug("Making URL Request: " + api_url)
        async with session.get(api_url) as response:
            if response.status == 200:
                LOGGER.info(
                    f"Got Open Weather API response : {str(await response.json())}"
                )
                resp = await response.json()
                return True, resp
            else:
                LOGGER.error(f"Error in making API Request : {str(response.json())}")
                return False, None



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py
@Path    :   agrothon/
@Time    :   2021/05/24
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.3.1
@Contact :   ckvbalusu@gmail.com
@Desc    :   Initialization Module for Agrothon
"""

__VERSION__ = "1.3.1"

import logging
import os
import time

if os.path.exists("Agrothon.txt"):
    with open("Agrothon.txt", "r+") as f:
        f.truncate(0)
from .server.helpers.database import MongoClient

try:
    from AgroConfig import Config
except ModuleNotFoundError:
    from agrothon.BaseConfig import Config

from logging.handlers import RotatingFileHandler

from .tgbot.translations import setLang

# Logger
logging.basicConfig(
    level=logging.DEBUG,
    format="%(asctime)s - %(levelname)s - %(message)s [%(filename)s:%(lineno)d]",
    datefmt="%d-%b-%y %H:%M:%S",
    handlers=[
        RotatingFileHandler("Agrothon.txt", maxBytes=50000000, backupCount=10),
        logging.StreamHandler(),
    ],
)
logging.getLogger("pyrogram").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)
LOGGER = logging.getLogger(__name__)


# Importing Variables from Config

SERVER_TIME = time.time()
TIME_Z = Config.TIME_ZONE
os.environ["TZ"] = Config.TIME_ZONE
time.tzset()
LANGUAGE = Config.LANGUAGE
OPEN_WEATHER_API = Config.OPEN_WEATHER_API
TELEGRAM_APP_ID = Config.TELEGRAM_APP_ID
TELEGRAM_API_HASH = Config.TELEGRAM_API_HASH
BOT_TOKEN = Config.BOT_TOKEN
LANG = setLang(LANGUAGE)
DEF_CITY = Config.DEF_CITY
API_KEY = Config.API_KEY
API_BASE_URL = Config.API_BASE_URL
if API_BASE_URL is not None:
    if not API_BASE_URL.endswith("/"):
        API_BASE_URL = API_BASE_URL + "/"
ALERT_CHANNEL_ID = Config.ALERT_CHANNEL_ID
DISTRICT = Config.DISTRICT
STATE = Config.STATE
DB_URL = Config.DB_URL
SENSOR_PRIORITY_INDEX = Config.SENSOR_PRIORITY_INDEX

# Commands
STATS_COMMAND = Config.STATS_COMMAND
FIELD_COMMAND = Config.FIELD_COMMAND
SETTINGS_COMMAND = Config.SETTINGS_COMMAND
WEATHER_COMMAND = Config.WEATHER_COMMAND
LOG_COMMAND = Config.LOG_COMMAND
HELP_COMMAND = Config.HELP_COMMAND
RAIN_COMMAND = Config.RAIN_COMMAND
RESTART_COMMAND = Config.RESTART_COMMAND
PING_COMMAND = Config.PING_COMMAND
AUTH_ID = Config.AUTH_ID

MDBClient = MongoClient(DB_URL)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   database.py
@Path    :   agrothon/server/helpers/
@Time    :   2021/05/5
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.2
@Contact :   ckvbalusu@gmail.com
@Desc    :   database module
"""

import logging

from motor.motor_asyncio import AsyncIOMotorClient

LOGGER = logging.getLogger(__name__)


class MongoClient:
    def __init__(self, DB_URL):
        self.DataBaseClient = None
        self.DB_URL = DB_URL

    async def get_db(self):
        """Return Database"""
        LOGGER.debug("Getting Database")
        return self.DataBaseClient.Agrothon

    async def connect_db(self):
        """Create database connection."""
        LOGGER.info("Connecting to database")
        self.DataBaseClient = AsyncIOMotorClient(self.DB_URL)

    async def close_mongo_connection(self):
        """Close database connection."""
        LOGGER.info("Closing database connection")
        self.DataBaseClient.close()



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py
@Path    :   agrothon/tgbot/translations/
@Time    :   2021/05/8
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.0
@Contact :   ckvbalusu@gmail.com
@Desc    :   Language translations init
"""

from . import english, hindi, tamil, telugu


def setLang(lang):
    if lang == "telugu":
        _lang = telugu.Language()
    elif lang == "tamil":
        _lang = tamil.Language()
    elif lang == "hindi":
        _lang = hindi.Language()
    elif lang == "english":
        _lang = english.Language()
    return _lang



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   AlertBot.py
@Path    :   agrothon/
@Time    :   2021/05/24
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   Intruder alert bot
"""
import os
import time
from logging import getLogger

from prettytable import PrettyTable
from telegram.ext import Updater

from agrothon import ALERT_CHANNEL_ID, BOT_TOKEN, LANG

from .tgbot.helpers.apiserverhelper import get_image_url, get_image_uuids

LOGGER = getLogger(__name__)
tg = Updater(token=BOT_TOKEN)
AlertBot = tg.bot


def alerts_handler():
    while True:
        response = get_image_uuids()
        if response is not None:
            for image in response["image_data"]:
                pt = PrettyTable([LANG.OBJECTS, LANG.DET_NO])
                pt.align[LANG.OBJECTS] = "l"
                pt.align[LANG.DET_NO] = "c"
                pt.padding_width = 0
                only_h = image["only_humans"]
                image_url = get_image_url(image["uuid"])
                detections = image["detections"]
                hums = image["humans"]
                tot_dets = image["no_of_detections"]
                for obj in detections:
                    pt.add_row([obj["type"], obj["count"]])
                at = image["at"]
                try:
                    if not only_h:
                        AlertBot.sendPhoto(
                            chat_id=ALERT_CHANNEL_ID,
                            photo=image_url,
                            caption=LANG.ALERT_MESSAGE.format(at, tot_dets, hums, pt),
                            parse_mode="HTML",
                        )
                    else:
                        AlertBot.sendPhoto(
                            chat_id=ALERT_CHANNEL_ID,
                            photo=image_url,
                            caption=LANG.ALERT_MESSAGE.format(at, tot_dets, hums, pt),
                            parse_mode="HTML",
                            disable_notification=True,
                        )
                except Exception as e:
                    LOGGER.error(
                        f"Error Occurred While Getting or Posting Alerts : {e}"
                    )
        time.sleep(1)


def restart_check():
    if os.path.isfile(".restartfile"):
        with open(".restartfile") as f:
            chat_id, msg_id = map(int, f)
        AlertBot.edit_message_text(LANG.RESTART_DONE, chat_id, msg_id)
        os.remove(".restartfile")


def language_change_check():
    if os.path.isfile(".setlangfile"):
        with open(".setlangfile") as f:
            chat_id, msg_id = map(int, f)
        AlertBot.edit_message_text(LANG.LANG_SET, chat_id, msg_id)
        os.remove(".setlangfile")



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   intruder.py
@Path    :   agrothon/server/routers/
@Time    :   2021/05/24
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   Intruder Image Detection Router using Yolov3
"""

import base64
import io
import logging
import uuid
from datetime import datetime
from uuid import UUID

import orjson
from fastapi import APIRouter, File, HTTPException, UploadFile
from fastapi.encoders import jsonable_encoder
from fastapi.responses import ORJSONResponse
from starlette.responses import StreamingResponse

from agrothon import MDBClient

from ..helpers.response_models import (
    ImageSuccessResponse,
    ImageUUID,
    IntruderInstantResponse,
)
from ..helpers.yolov3_helper import yolo_detect

LOGGER = logging.getLogger(__name__)

IntruderRouter = APIRouter(
    prefix="/intruder",
    tags=["Intruder Detection"],
    responses={404: {"error": "Not found"}},
)


@IntruderRouter.post(
    "/detect/instant",
    response_class=ORJSONResponse,
    response_model=IntruderInstantResponse,
)
async def image_instant_detect_post(image: UploadFile = File(...)):
    image_cont = await image.read()
    _db = await MDBClient.get_db()
    _intruder_instant = _db["instant_detect"]
    n_docs = await _intruder_instant.estimated_document_count()
    status, nos, det_list, _hum, _only_hum, _img = yolo_detect(image_cont, filter=False)
    if status is not None:
        if status:
            _uuid = str(uuid.uuid4())
            b64_image_string = base64.b64encode(_img)
            await _intruder_instant.update_one(
                {"_id": n_docs},
                {"$set": {"uuid": _uuid, "image": b64_image_string}},
                upsert=True,
            )
            LOGGER.info(f"Saved an image in database with UUID {_uuid}")
            detections_dict = {
                "uuid": _uuid,
                "detections": det_list,
                "no_of_detections": nos,
                "humans": _hum,
                "only_humans": _only_hum,
            }
            LOGGER.debug(
                f"Instant Image Detection Response : {orjson.dumps(detections_dict)}"
            )
            return ORJSONResponse(content=detections_dict)
        else:
            LOGGER.warning("Nothing found in the image")
            raise HTTPException(
                status_code=420, detail={"Error": "Nothing found in image"}
            )
    else:
        LOGGER.error(f"Something wrong happened with Yolov3")
        raise HTTPException(
            status_code=404, detail={"Error": "Error occoured, try again"}
        )


@IntruderRouter.get("/detect/instant/{image_uuid}", response_class=StreamingResponse)
async def image_instant_get(image_uuid: UUID):
    """
    - **image_uuid** : Unique UUID to Search image in Database
    * Response will be the Image
        * Once the image is fetched it will be deleted from the database

    """
    _db = await MDBClient.get_db()
    _intruder_instant = _db["instant_detect"]
    try:
        LOGGER.info(f"Searching in Database for the IMage with UUID {image_uuid}")
        data = await _intruder_instant.find_one_and_delete({"uuid": str(image_uuid)})
        image = base64.b64decode(data["image"])
        return StreamingResponse(io.BytesIO(image), media_type="image/jpeg")
    except TypeError as e:
        LOGGER.warning(f"Image not found in the database : Error: {e}")
        raise HTTPException(
            status_code=404,
            detail={"status": f"Image UUID {image_uuid} not found in the DB"},
        )


@IntruderRouter.post(
    "/detect", response_class=ORJSONResponse, response_model=ImageSuccessResponse
)
async def image_post(image: UploadFile = File(...)):
    """
    - **Post an image*
    - **image** : Image
    - This image will be analyzed for intruders and returns uuid of image
    - if Nothing is found error 420 is returned

    """
    _db = await MDBClient.get_db()
    _intruder = _db["intruder"]
    _intruder_images = _db["images"]
    now = datetime.now()
    time_date = now.strftime("%X %x")
    image_cont = await image.read()
    status, nos, det_list, _hum, _only_hum, _img = yolo_detect(image_cont)
    n_docs = await _intruder.estimated_document_count()
    if status is not None:
        if status:
            _uuid = str(uuid.uuid4())
            b64_image_string = base64.b64encode(_img)
            detections_dict = {
                "uuid": _uuid,
                "detections": det_list,
                "no_of_detections": nos,
                "humans": _hum,
                "only_humans": _only_hum,
                "at": time_date,
            }
            image_dict = {"uuid": _uuid, "image": b64_image_string}
            await _intruder.update_one(
                {"_id": n_docs}, {"$set": detections_dict}, upsert=True
            )
            await _intruder_images.update_one(
                {"_id": n_docs}, {"$set": image_dict}, upsert=True
            )
            LOGGER.info(f"Saved an image in database with UUID {_uuid}")
            j_resp = {"Detected_Intruders": True, "image_uuid": _uuid}
            LOGGER.debug(f"no of intruders detected: {str(nos)}")
            return ORJSONResponse(content=j_resp)
        else:
            LOGGER.warning("Nothing found in the image")
            raise HTTPException(
                status_code=420, detail={"Error": "Nothing found in image"}
            )
    else:
        LOGGER.error(f"Something wrong happened with Yolov3")
        raise HTTPException(
            status_code=404, detail={"Error": "Error occoured, try again"}
        )


@IntruderRouter.get("/detect/image/{image_uuid}", response_class=StreamingResponse)
async def image_get(image_uuid: UUID):
    """
    - **image_uuid** : Unique UUID to Search image in Database
    * Response will be the Image
        * Once the image is fetched it will be deleted from the database

    """
    _db = await MDBClient.get_db()
    _intruder = _db["intruder"]
    _intruder_images = _db["images"]
    try:
        data = await _intruder_images.find_one_and_delete({"uuid": str(image_uuid)})
        await _intruder.delete_one({"uuid": str(image_uuid)})
        image = base64.b64decode(data["image"])
        return StreamingResponse(io.BytesIO(image), media_type="image/jpeg")
    except KeyError as e:
        LOGGER.warning(f"Image not found in the database, Error : {e}")
        raise HTTPException(
            status_code=404,
            detail={"status": f"Image UUID {image_uuid} not found in the DB"},
        )


@IntruderRouter.get(
    "/images/uuids", response_class=ORJSONResponse, response_model=ImageUUID
)
async def uuids_get():
    """
    get all the available image UUID's in the Database according to the Priority

    """
    LOGGER.debug(f"Getting all the UUID's in the database")
    _db = await MDBClient.get_db()
    _intruder = _db["intruder"]
    n_docs = await _intruder.estimated_document_count()
    image_data = []
    if n_docs != 0:
        LOGGER.info(f"No of Pending Alerts found : {n_docs}")
        async for doc in _intruder.find({}, {"_id": False}).sort("_id").limit(n_docs):
            j_resp = jsonable_encoder(doc)
            image_data.append(j_resp)
        resp_dict = {"pending_alerts": n_docs, "image_data": image_data}
        return ORJSONResponse(content=resp_dict)
    else:
        LOGGER.warning(f"Nothing Found in the Database")
        raise HTTPException(
            status_code=404, detail={"status": "LMAO Nothing found dude"}
        )


@IntruderRouter.get(
    "/images/uuid/{image_uuid}",
    response_class=ORJSONResponse,
    response_model=IntruderInstantResponse,
)
async def get_using_uuid(image_uuid: UUID):
    """
    get all the available image UUID's in the Database according to the Priority

    """
    LOGGER.debug(f"Getting all the UUID's in the database")
    _db = await MDBClient.get_db()
    _intruder = _db["intruder"]
    data = await _intruder.find_one({"uuid": str(image_uuid)}, {"_id": False})
    if data is not None:
        j_resp = jsonable_encoder(data)
        LOGGER.debug(f"Data fetched using UUID {str(image_uuid)}: {j_resp}")
        return ORJSONResponse(content=j_resp)
    else:
        LOGGER.warning(f"Nothing Found in the Database")
        raise HTTPException(
            status_code=404, detail={"status": "LMAO Nothing found dude"}
        )



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   telugu.py
@Path    :   agrothon/tgbot/translations/
@Time    :   2021/05/24
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   Telugu translations
"""


class Language(object):
    # Main Menu
    MAIN_MENU = "<b>ప్రధాన మెనూ</b>"

    # Open Weather
    WEATHER_FETCH = "వివరాలు శోధిస్తోంది దయచేసి వేచి ఉండండి"
    WEATHER = """
<b>📍 స్థానం :</b><code> {}</code>
<b>🌡️ ఉష్ణోగ్రత : </b><code>{} °C</code>
<b>💨 ప్రెజర్ : </b><code>{} Pa</code>
<b>💧 తేమ : </b><code>{} g.m-3</code>
<b>⛅ వాతావరణం : </b><code>{}</code>
"""
    WEATHER_ERR = "<b>లోపం సంభవించింది, ప్రతిస్పందన: <code>{}</code> </b>"

    # Button Callbacks

    MOISTURE = "తేమ"
    HUMIDITY = "తడి"
    TEMPERATURE = "ఉష్ణోగ్రత"
    RAIN = "వర్షపాతం స్థితి"
    PUMP_STATUS = "పంప్ స్థితి"
    COMPLETE_INFO = "పూర్తి సమాచారం"
    QUIT = "క్విట్"
    PUMP_OFF = "ఆపి వేయి"
    PUMP_ON = "స్విచ్ ఆన్"
    BACK = "వెనుక మెను"
    BOT_PRED = "బొట్ ప్రిడిక్షన్ ఆన్ చేయండి"
    REFRESH = "స్థితిని రిఫ్రెష్"

    # Thing Speak Data
    MOISTURE_SENSOR = """<b>💧 మట్టిలో తేమ (సెన్సార్ {}): </b><code> {}%</code>\n"""
    MOISTURE_RESP = """<b>🕒 చివరిసారి నవీకరించబడింది : </b><code> {}</code>
<b>🕒 చివరి తనిఖీ : </b><code> {}</code>
"""
    HUMID_RESP = """
<b>⛅ పొలంలో తేమ : </b><code> {}%</code>
<b>🕒 చివరిసారి నవీకరించబడింది : </b><code> {}</code>
<b>🕒 చివరి తనిఖీ : </b><code> {}</code>
"""
    TEMPE_RESP = """
<b>🌡️ పొలంలో ఉష్ణోగ్రత : </b><code> {}°C</code>
<b>🕒 చివరిసారి నవీకరించబడింది: </b><code> {}</code>
<b>🕒 చివరి తనిఖీ: </b><code> {}</code>
"""
    RAIN_YES_RESP = """
<b>పొలంలో వర్షం పడుతోంది 🌧️</b>

"""
    RAIN_NO_RESP = """
<b>పొలంలో వర్షం పడటం లేదు 🌞</b>

"""
    COMPLETE_MOISTURE = """<b>💧 తేమ (సెన్సార్ {}): </b><code> {}%</code>\n"""
    COMPLETE_RESP = """<b>⛅ తడి: </b><code> {}%</code>
<b>🌡️ ఉష్ణోగ్రత : </b><code> {}°C</code>
<b>⛏️ పంప్ స్థితి ఎలా ఉండాలి (సెన్సార్ {}): </b><code> {}</code>
<b>🕒 చివరిసారి నవీకరించబడింది : </b><code> {}</code>
<b>🕒 చివరి తనిఖీ : </b><code> {}</code>
"""
    # Pump
    PUMP_SWITCHED_ON = """
<b>పంప్ ఆన్లో ఉంది</b>
<code>{} </code> ద్వారా మార్చబడింది
చివరి తనిఖీ : <code> {}</code>
"""

    PUMP_SWITCHED_OFF = """
<b>పంప్ ఆఫ్లో ఉంది</b>
<code>{} </code> ద్వారా స్విచ్ ఆఫ్ చేయబడింది
చివరి తనిఖీ : <code> {}</code>
"""

    BOT_ACTIVATED = """<b>🤖 బాట్ మోడ్ సక్రియం చేయబడింది </b>
ఇప్పుడు మీరు తిరిగి కూర్చుని విశ్రాంతి తీసుకోవచ్చు 🛏️ మరియు మీ 🤖 బొట్ మీ పొలాన్ని నిర్వహించడానికి అనుమతించండి 🚜
"""
    PUMP_BTN_ON = """
<b> ✅ రన్నింగ్ స్థితి మార్చబడింది</b>
<code>User </code> ద్వారా మార్చబడింది
"""

    PUMP_BTN_OFF = """
<b> ✅ రన్నింగ్ స్థితి మార్చబడింది </b>
<code>User </code> ద్వారా మార్చబడింది
"""
    LANG = "🌐 భాష మార్చు 🌐"
    SETTINGS = "⚙️ సెట్టింగులు ⚙️"
    SELECT_LANG = "ప్రాధాన్య భాష"
    LANG_CHANGED = "భాష మారుతోంది, దయచేసి వేచి ఉండండి"
    OBJECTS = "వస్తువులు"
    DET_NO = "కనుగొన్నారు"
    ALERT_MESSAGE = """
<b>చొరబాటుదారులు కనుగొనబడ్డారు </b>
<b>కనుగొనబడిన సమయం</b> : <code> {}</code>
<b>వస్తువుల సంఖ్య</b> : <code> {}</code>
<b>వ్యక్తుల సంఖ్య</b> : <code> {}</code>

<pre>{}</pre>
"""
    MONTHS = [
        "ఏప్రిల్",
        "మే",
        "జూన్",
        "జూలై",
        "ఆగస్టు",
        "సెప్టెంబర్",
        "అక్టోబర్",
        "నవంబర్",
        "డిసెంబర్",
    ]
    MONTH = "నెల"
    RAINFALL = "వర్షపాతం ({} లో)"
    RAIN_PREDICT = """
<b> ఈ సంవత్సరానికి వర్షపాతం అంచనాలు </b>
<b> రాష్ట్రం </b>: <code> {} </code>
<b> జిల్లా </b>: <code> {} </code>

<pre>{}</pre>
"""
    RAIN_PREDICT_ERR = """
ప్రిడిక్షన్ చేస్తున్నప్పుడు లోపం
"""
    IMAGE_MESSAGE = """
<b>వస్తువులు కనుగొనబడ్డాయి</b>
<b>వస్తువుల సంఖ్య</b> : <code> {}</code>
<b>వ్యక్తుల సంఖ్య</b> : <code> {}</code>

<pre>{}</pre>
"""
    ERR_IMAGE_RESPONSE = """
<b>చిత్రంలో ఏమీ కనుగొనబడలేదు</b>
"""
    PRED_PUMP_OFF = "ఆఫ్"
    PRED_PUMP_ON = "ఆన్"
    STATS = """
<b> సమయము: </b> <code> {} </code>
<b> డిస్క్ స్థలం: </b> <code> {} </code>
<b> ఉపయోగించబడింది: </b> <code> {} </code>
<b> ఉచిత: </b> <code> {} </code>
<b> CPU వినియోగం: </b> <code> {}% </code>
<b> RAM వినియోగం: </b> <code> {}% </code>
<b> అప్‌లోడ్ చేయబడింది: </b> <code> {} </code>
<b> డౌన్‌లోడ్ చేయబడింది: </b> <code> {} </code>
"""
    DL_TG = "టెలిగ్రామ్ నుండి డౌన్‌లోడ్ అవుతోంది"
    PROC_IMAGE = "డౌన్‌లోడ్ చేయబడింది, వస్తువులను గుర్తించడం దయచేసి వేచి ఉండండి ..."
    RESTART = "పునఃప్రారంభించి, దయచేసి వేచి ...."
    RESTART_DONE = "విజయవంతంగా పునఃప్రారంభం!"
    RESTART_CALLBACK = "పునఃప్రారంభించండి"
    WEATHER_FETCHING = "వాతావరణం పొందడం, దయచేసి వేచి ఉండండి"
    HELP_MESSAGE = """
<code>/{} </code>: మీ నగరం యొక్క వాతావరణ స్థితి
<code>/{} </code>: మీ ప్రాంతం యొక్క వర్షపాతాన్ని అంచనా వేయండి
<code>/{} </code>: మీ ఫీల్డ్ స్థితిని పొందండి మరియు మీ పంపుని నిర్వహించండి
<code>/{} </code>: మీ బొట్ యొక్క సెట్టింగులను మార్చండి
<code>/{} </code>: సర్వర్ గణాంకాలను పొందండి
<code>/{} </code>: పింగ్‌ను తనిఖీ చేయండి
<code>/{} </code>: సర్వర్ యొక్క లాగ్ పొందండి
<code>/{} </code>: సర్వర్‌ను పున ప్రారంభించండి
<code>/{} </code>: ఈ సందేశాన్ని పొందడానికి

<code> చిత్రంలోని వస్తువులను గుర్తించడానికి చిత్రాన్ని పంపండి </code>
"""
    START = """
హే, నేను <code> అగ్రోథాన్ </code>

  - మీరు మీ పొలాన్ని పర్యవేక్షించవచ్చు
  - పంప్ స్థితిని ఆన్ లేదా ఆఫ్‌కు మార్చండి
  - వాతావరణం పొందండి
  - చిత్రంలోని వస్తువులను గుర్తించండి
  - మీ ప్రాంతానికి వర్షపాతాన్ని అంచనా వేయండి
"""
    PING_START = "పింగ్ ప్రారంభమవుతుంది"
    PING_FINAL = "కొలిచిన పింగ్ : {}"
    LANG_SET = "విజయవంతంగా భాషను తెలుగుగా మార్చారు"
    PUMP_STATUS_ON = "ఆన్"
    PUMP_STATUS_OFF = "ఆఫ్"



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   pump_prediction.py
@Path    :   agrothon/server/helpers/
@Time    :   2021/05/5
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.2
@Contact :   ckvbalusu@gmail.com
@Desc    :   Pump Prediction Module
"""
import logging
from typing import Optional

import joblib
import numpy as np

pump_model = joblib.load("data/models/pump/pump.sav")

LOGGER = logging.getLogger(__name__)


async def predict_pump(
    moisture: float, temperature: float, humidity: float
) -> Optional[bool]:
    try:
        LOGGER.info(
            f"Predicting Pump using moisture: {moisture}, temperature: {temperature}, humidity: {humidity}"
        )
        prediction = pump_model.predict(np.array([[moisture, temperature, humidity]]))
        return bool(prediction[0])
    except Exception as e:
        LOGGER.error(f"Error Occured while predicting pump, Error : {e}")
        return None



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   sensors.py
@Path    :   agrothon/server/routers/
@Time    :   2021/05/5
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   Sensors data post or get Routers
"""
import logging
from datetime import datetime

import orjson
from fastapi import APIRouter, HTTPException, status
from fastapi.encoders import jsonable_encoder
from fastapi.responses import ORJSONResponse

from agrothon import SENSOR_PRIORITY_INDEX, MDBClient

from ..helpers.pump_prediction import predict_pump
from ..helpers.response_models import (
    Sensor,
    SensorAll,
    SensorData,
    SuccessResponseSensor,
)

LOGGER = logging.getLogger(__name__)


SensorRouter = APIRouter(
    prefix="/field",
    tags=["Field Data"],
    responses={404: {"error": "Not found"}},
)


@SensorRouter.get("/sensor", response_class=ORJSONResponse, response_model=Sensor)
async def sensor_get():
    """
    Get the Last Update of the sensor data
    """
    LOGGER.info(f"Getting Sensor data from the database")
    _db = await MDBClient.get_db()
    _sensor = _db["sensor"]
    now = datetime.now()
    time_date = now.strftime("%X %x")
    try:
        n_docs = await _sensor.estimated_document_count()
        latest_doc_id = int(n_docs) - 1
        data = await _sensor.find_one({"_id": latest_doc_id}, {"_id": False})
        await _sensor.update_one(
            {"_id": latest_doc_id}, {"$set": {"last_read": time_date}}
        )
        j_resp = jsonable_encoder(data)
        LOGGER.debug(f"Fetched sensor data : {j_resp}")
        return ORJSONResponse(content=j_resp)
    except Exception as e:
        LOGGER.error(f"Error Occurred While getting data from database, Error: {e}")
        raise HTTPException(status_code=404, detail={"error": "Nothing Found"})


@SensorRouter.get(
    "/sensor/all", response_class=ORJSONResponse, response_model=SensorAll
)
async def sensor_get_all():
    """
    **Get all the Sensor data from the database**
    """
    LOGGER.info("Getting all the available sensor data from database")
    _db = await MDBClient.get_db()
    _sensor = _db["sensor"]
    n_docs = await _sensor.estimated_document_count()
    sensor_data = []
    if n_docs != 0:
        async for doc in _sensor.find({}, {"_id": False, "last_read": False}).sort(
            "_id"
        ).limit(n_docs):
            sensor_data.append(jsonable_encoder(doc))
        LOGGER.info(f"Fetched {n_docs} entries of sensor data")
        return ORJSONResponse(
            content={"no_of_entries": n_docs, "sensor_data": sensor_data}
        )
    else:
        raise HTTPException(status_code=404, detail={"error": "Nothing Found"})


@SensorRouter.post(
    "/sensor", response_class=ORJSONResponse, response_model=SuccessResponseSensor
)
async def sensor_post(data: SensorData):
    """
    **post sensor data from the field to DB**
    - Accepts Temperature, Humidity, Temperature as JSON
    """
    LOGGER.info(f"Parsing Sensor data that was posted")
    _db = await MDBClient.get_db()
    _sensor = _db["sensor"]
    _pump = _db["pump"]
    now = datetime.now()
    time_date: str = now.strftime("%X %x")
    pump_data = await _pump.find_one({"_id": "pump"}, {"_id": False})
    pump_set = False
    pump_stat = await predict_pump(
        data.moisture[SENSOR_PRIORITY_INDEX - 1], data.temperature, data.humidity
    )
    try:
        try:
            if pump_data["by"] == "AI Bot":
                if pump_data["status"]:
                    if pump_stat:
                        pump_set = False
                    else:
                        pump_set = True
                else:
                    if pump_stat:
                        pump_set = True
                    else:
                        pump_set = False
            else:
                pump_set = False
        except TypeError:
            pump_set = True
        n_docs = await _sensor.estimated_document_count()
        new_doc_id = int(n_docs)
        data_dict = {
            "no_of_sensors": data.no_of_sensors,
            "moisture": data.moisture,
            "humidity": data.humidity,
            "temperature": data.temperature,
            "pump_prediction": pump_stat,
            "sensor_priority": SENSOR_PRIORITY_INDEX,
            "updated_at": time_date,
            "last_read": time_date,
        }
        await _sensor.update_one(
            {"_id": new_doc_id},
            {"$set": data_dict},
            upsert=True,
        )
        if pump_set:
            pump_dict = {
                "status": pump_stat,
                "time": time_date,
                "by": "AI Bot",
                "last_read": time_date,
            }
            await _pump.update_one(
                {"_id": "pump"},
                {"$set": pump_dict},
                upsert=True,
            )
            LOGGER.debug(
                f"Pump Status has been updated : {str(orjson.dumps(pump_dict))}"
            )
        j_resp = {"Success": True, "pump_status_updated": pump_set}
        LOGGER.debug(f"Sensor Post response : {str(j_resp)}")
        return ORJSONResponse(content=j_resp)
    except Exception as e:
        LOGGER.error(f"Error Occurred while Updating :{e}")
        j_resp = jsonable_encoder({"Success": False})
        return ORJSONResponse(
            status_code=status.HTTP_422_UNPROCESSABLE_ENTITY, content=j_resp
        )



from setuptools import setup, find_packages
from agrothon import __VERSION__ as VER
import pathlib

CWD = pathlib.Path(__file__).parent

README = (CWD / "README.md").read_text()

setup(
    name='Agrothon',
    version=VER,
    packages=find_packages(),
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/viswanathbalusu/agrothon',
    license='GPL3.0',
    author='viswanathbalusu',
    author_email='ckvbalusu@gmail.com',
    include_package_data=True,
    description='A Farm Monitoring Bot',
    platforms="any",
    install_requires=[
        "pyrogram==1.2.9",
        "tgcrypto==1.2.2",
        "motor==2.4.0",
        "fastapi[all]==0.64.0",
        "requests==2.25.1",
        "python-dotenv==0.17.0",
        "aiohttp==3.7.4.post0",
        "python-telegram-bot==13.5",
        "pandas==1.2.4",
        "numpy==1.19.5",
        "tensorflow==2.4.1",
        "prettytable==2.1.0",
        "opencv-python==4.5.1.48",
        "pillow==8.2.0",
        "seaborn==0.11.1",
        "Keras==2.4.3",
        "scikit-learn==0.22.2.post1",
        "telegraph==1.4.1",
        "psutil==5.8.0"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: POSIX :: Linux",
        "Development Status :: 5 - Production/Stable"
    ],
    python_requires=">=3.8",
    entry_points={
        "console_scripts":[
            "agrobot = agrothon.__main__:main"
        ]

    },
    scripts=['bin/agroserver', 'bin/agrothon'],
)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   hindi.py
@Path    :   agrothon/tgbot/translations/
@Time    :   2021/05/24
@Author  :   github.com/xavierxross
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   Hindi translations
"""


class Language(object):
    # Main Menu
    MAIN_MENU = "<b>मुख्य मेन्यू</b>"

    # Open Weather
    WEATHER_FETCH = "कृपया प्रतीक्षा करें जानकारी के लिये...."
    WEATHER = """
<b>📍 स्थान :</b><code> {}</code>
<b>🌡️ तापमान : </b><code>{} °C</code>
<b>💨 दबाव/प्रिशर : </b><code>{} Pa</code>
<b>💧 आर्द्रता : </b><code>{} g.m-3</code>
<b>⛅ मौसम : </b><code>{}</code>
"""
    WEATHER_ERR = "<b>गड़बड़ी हुइ हे, प्रतिक्रिया : <code>{}</code> </b>"

    # Button Callbacks

    MOISTURE = "नमी"
    HUMIDITY = "आर्द्रता"
    TEMPERATURE = "तापमान"
    RAIN = "वर्षा स्थिति"
    PUMP_STATUS = "पंप स्थिति"
    COMPLETE_INFO = "पूर्ण जानकारी"
    QUIT = "बंद करे"
    PUMP_OFF = "स्विच ऑफ"
    PUMP_ON = "स्विच वन"
    BACK = "पीछे"
    BOT_PRED = "बौट पूर्वानुमान को चालू करें"
    REFRESH = "रिफ्रेश स्थिति"
    MOISTURE_SENSOR = """<b>💧 मिट्टी में नमी (सेंसर {}): </b><code> {}%</code>\n"""
    MOISTURE_RESP = """<b>🕒 अंतिम बार अपडेट किया गया: </b><code> {}</code>
<b>🕒 अंतिम बार देखा गया था: </b><code> {}</code>
"""
    HUMID_RESP = """
<b>⛅ मैदान में आर्द्रता : </b><code> {}%</code>
<b>🕒 अंतिम बार अपडेट किया गया: </b><code> {}</code>
<b>🕒 अंतिम बार देखा गया था: </b><code> {}</code>
"""
    TEMPE_RESP = """
<b>मैदान में तापमान🌡️ : </b><code> {}°C</code>
<b>🕒 अंतिम बार अपडेट किया गया: </b><code> {}</code>
<b>🕒 अंतिम बार देखा गया था: </b><code> {}</code>
"""
    RAIN_YES_RESP = """
<b>वहां बारिश🌧️ हो रही है</b>

"""
    RAIN_NO_RESP = """
<b>बारिश नहीं हो रही है 🌞</b>

"""
    COMPLETE_MOISTURE = """<b>💧 नमी (सेंसर {}): </b><code> {}%</code>\n"""
    COMPLETE_RESP = """<b>⛅ आर्द्रता : </b><code> {}%</code>
<b>🌡️ तापमान : </b><code> {}°C</code>
<b>⛏️ पंप होना चाहिए (सेंसर {}): </b><code> {}</code>
<b>🕒 अंतिम बार अपडेट किया गया: </b><code> {}</code>
<b>🕒 अंतिम बार देखा गया था: : </b><code> {}</code>
"""
    # Pump
    PUMP_SWITCHED_ON = """
<b>पंप चल रहा है</b>
स्विच वन किया <code> {}</code>
अंतिम जांच : <code> {}</code>
"""

    PUMP_SWITCHED_OFF = """
<b>पंप बंद है</b>
स्विच ऑफ किया  <code> {}</code>
अंतिम जांच : <code> {}</code>
"""

    BOT_ACTIVATED = """<b>🤖 बॉट मोड सक्रिय है </b>
अब आप आराम से बैठ सकते हैं 🛏️ और 🤖 बॉट को अपने खेत को संभालने दें 🚜
"""
    PUMP_BTN_ON = """
<b> ✅ चलने का स्थिति बदल दिया </b>
स्विच वन किया <code> User</code>
"""

    PUMP_BTN_OFF = """
<b> ✅ चलने का स्थिति बदल दिया </b>
स्विच ऑफ किया <code> User</code>
"""

    SETTINGS = "⚙️ सेटिंग ⚙️"
    LANG = "🌐 भाषा बदलें 🌐"
    SELECT_LANG = "पसंदीदा भाषा का चयन करें"
    LANG_CHANGED = "भाषा बदलने की प्रतीक्षा करें"
    OBJECTS = "वस्तुओं"
    DET_NO = "मिल गया"
    ALERT_MESSAGE = """
<b>घुसपैठियों का पता चला है </b>
<b>येहा पता चला हे</b> : <code> {}</code>
<b>वस्तुओं की संख्या पता लगाना</b> : <code> {}</code>
<b>लोगों की संख्या पता लगाना</b> : <code> {}</code>

<pre>{}</pre>
"""
    MONTHS = [
        "अप्रैल",
        "मई",
        "जून",
        "जुलाई",
        "अगस्त",
        "सितम्बर",
        "अक्टूबर",
        "नवम्बर",
        "दिसम्बर",
    ]
    MONTH = "महीना"
    RAINFALL = "वर्षा(in {})"
    RAIN_PREDICT = """
<b>वर्षा पूर्वानुमान इस साल के लिए</b>
<b>राज्य </b>: <code> {}</code>
<b>जिला </b>: <code> {}</code>

<pre>{}</pre>
"""
    RAIN_PREDICT_ERR = """
गड़बड़ी जबकि
"""
    IMAGE_MESSAGE = """
<b>वस्तुओं या लोगों का पता चला है </b>
<b>वस्तुओं की संख्या पता लगाना</b> : <code> {}</code>
<b>लोगों की संख्या पता लगाना</b> : <code> {}</code>

<pre>{}</pre>
"""
    ERR_IMAGE_RESPONSE = """
<b>छवि में कुछ भी नहीं मिला</b>
"""
    PRED_PUMP_OFF = "ऑफ"
    PRED_PUMP_ON = "वन"
    STATS = """
<b>अपटाइम:</b><code> {}</code>
<b>डिस्क स्थान :</b><code> {}</code>
<b>प्रयुक्त :</b><code> {}</code>
<b>निःशुल्क :</b><code> {}</code>
<b>CPU उपयोग:</b><code> {}%</code>
<b>RAM :</b><code> {}%</code>
<b>अपलोड किया गया:</b><code> {}</code>
<b>डाउनलोड किया गया:</b><code> {}</code>
"""
    DL_TG = "टेलीग्राम से डाउनलोड करना"
    PROC_IMAGE = "डाउनलोड किया गया, वस्तुओं का पता लगाना कृपया प्रतीक्षा करें..."
    RESTART = "पुनः प्रारंभ हो रहा है, कृपया प्रतीक्षा करें...."
    RESTART_DONE = "सफलतापूर्वक पुनरारंभ किया गया!"
    RESTART_CALLBACK = "पुनः आरंभ करें"
    WEATHER_FETCHING = "मौसम आ रहा है, कृपया प्रतीक्षा करें"
    HELP_MESSAGE = """
<code>/{}</code> : आपके शहर की मौसम स्थिति
<code>/{}</code> : अपने क्षेत्र की वर्षा की भविष्यवाणी करें
<code>/{}</code> : अपने क्षेत्र की स्थिति प्राप्त करें और अपने पंप का प्रबंधन करें
<code>/{}</code> : अपने बॉट की सेटिंग बदलें
<code>/{}</code> : सर्वर आँकड़े प्राप्त करें
<code>/{}</code> : पिंग चेक करें
<code>/{}</code> : सर्वर का लॉग प्राप्त करें
<code>/{}</code> : सर्वर को पुनरारंभ करें
<code>/{}</code> : यह संदेश पाने के लिए

<code>किसी इमेज में ऑब्जेक्ट का पता लगाने के लिए बस इमेज भेजें</code>
"""
    START = """
अरे, मैं हूं <code>एग्रोथॉन</code>

  - आप अपने खेत की निगरानी कर सकते हैं
  - पंप की स्थिति को चालू या बंद में बदलें
  - मौसम प्राप्त करें
  - एक छवि में वस्तुओं का पता लगाएं
  - अपने क्षेत्र के लिए वर्षा की भविष्यवाणी करें
"""
    PING_START = "पिंग शुरू करना"
    PING_FINAL = "मापा पिंग : {}"
    LANG_SET = "हिंदी को पसंदीदा भाषा के रूप में सेट करना सफल रहा"
    PUMP_STATUS_ON = "चालू"
    PUMP_STATUS_OFF = "बंद"



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   response_models.py
@Path    :   agrothon/server/helpers/
@Time    :   2021/05/24
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.2.7
@Contact :   ckvbalusu@gmail.com
@Desc    :   Response and Accept Models
"""
from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel


class SuccessResponseSensor(BaseModel):
    Success: bool
    pump_status_updated: bool


class SuccessResponse(BaseModel):
    Success: bool


class PumpGetResponse(BaseModel):
    status: bool
    by: str
    time: str
    last_read: Optional[str] = None


class PumpPostIN(BaseModel):
    status: bool
    by: str

    class Config:
        schema_extra = {
            "example": {
                "status": True,
                "by": "A Person",
            }
        }


class ImageSuccessResponse(BaseModel):
    Detected_Intruders: bool
    uuid: UUID


class Weather(BaseModel):
    city: str
    temperature: int
    pressure: int
    humidity: int
    weather: str


class Sensor(BaseModel):
    no_of_sensors: int
    moisture: List[float]
    humidity: float
    temperature: float
    updated_at: str
    pump_prediction: bool
    sensor_priority: int
    last_read: Optional[str] = None


class SensorData(BaseModel):
    no_of_sensors: int
    moisture: List[float]
    humidity: float
    temperature: float

    class Config:
        schema_extra = {
            "example": {
                "no_of_sensors": 2,
                "moisture": [19.25, 56.25],
                "humidity": 20.95,
                "temperature": 35,
            }
        }


class SensorBase(BaseModel):
    no_of_sensors: int
    moisture: List[float]
    humidity: int
    temperature: int
    pump_prediction: bool
    sensor_priority: int
    updated_at: str

    class Config:
        schema_extra = {
            "example": {
                "no_of_sensors": 2,
                "moisture": [19.25, 56.25],
                "humidity": 20.95,
                "temperature": 35,
                "pump_prediction": True,
                "sensor_priority": 1,
                "updated_at": "15:35:53 05/02/21",
            }
        }


class SensorAll(BaseModel):
    no_of_entries: int
    sensor_data: List[SensorBase]


class IntruderDetections(BaseModel):
    type: str
    confidences: List[float]
    count: int


class IntruderInstantResponse(BaseModel):
    uuid: UUID
    detections: List[IntruderDetections]
    no_of_detections: int
    humans: int
    only_humans: bool
    at: str


class ImageUUID(BaseModel):
    pending_alerts: int
    image_data: List[IntruderInstantResponse]


class Region(BaseModel):
    state: str
    district: str

    class Config:
        schema_extra = {
            "example": {
                "state": "Andhra Pradesh",
                "district": "East Godavari",
            }
        }


class MonthWisePrediction(BaseModel):
    april: float
    may: float
    june: float
    july: float
    august: float
    september: float
    october: float
    november: float
    december: float


class RainPredictionOut(BaseModel):
    state: str
    district: str
    units: str
    predictions: MonthWisePrediction



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   pump.py
@Path    :   agrothon/server/routers/
@Time    :   2021/05/3
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.0.0
@Contact :   ckvbalusu@gmail.com
@Desc    :   Pump on or off Router
"""
import logging
from datetime import datetime

from fastapi import APIRouter, HTTPException
from fastapi.encoders import jsonable_encoder
from fastapi.responses import ORJSONResponse

from agrothon import MDBClient

from ..helpers.response_models import PumpGetResponse, PumpPostIN, SuccessResponse

LOGGER = logging.getLogger(__name__)


PumpRouter = APIRouter(
    prefix="/pump",
    tags=["Pump"],
    responses={404: {"error": "Not found"}},
)


@PumpRouter.get("/", response_class=ORJSONResponse, response_model=PumpGetResponse)
async def pump_get():
    """
    **Pump Status**
    - Will return the present status  of the pump in the field
    """
    LOGGER.info(f"Getting Pump status")
    _db = await MDBClient.get_db()
    _pump = _db["pump"]
    now = datetime.now()
    time_date = now.strftime("%X %x")
    try:
        data = await _pump.find_one({"_id": "pump"}, {"_id": False})
        if data is not None:
            await _pump.update_one(
                {"_id": "pump"}, {"$set": {"last_read": time_date}}, upsert=True
            )
            j_resp = jsonable_encoder(data)
            LOGGER.debug(f"Pump status : {str(j_resp)}")
            return ORJSONResponse(content=j_resp)
        else:
            raise HTTPException(
                status_code=404, detail={"error": "Nothing found to Query"}
            )
    except AttributeError as e:
        LOGGER.error(f"Error Occurred while getting Pump Status : {e}")
        raise HTTPException(
            status_code=404, detail={"error": "Something wrong with the database"}
        )


@PumpRouter.post("/", response_class=ORJSONResponse, response_model=SuccessResponse)
async def pump_post(status: PumpPostIN):
    """
    **Update Pump Status in the field**
    """
    LOGGER.info(f"Updating Pump Status")
    _db = await MDBClient.get_db()
    _pump = _db["pump"]
    now = datetime.now()
    time_date = now.strftime("%X %x")
    try:
        await _pump.update_one(
            {"_id": "pump"},
            {
                "$set": {
                    "status": status.status,
                    "time": time_date,
                    "by": status.by,
                    "last_read": time_date,
                }
            },
            upsert=True,
        )
        j_resp = jsonable_encoder({"Success": True})
        return ORJSONResponse(content=j_resp)
    except Exception as e:
        LOGGER.error(f"Error Occurred while Updating Pump Status : {e}")
        raise HTTPException(status_code=404, detail={"error": "Cant Post"})



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   rainfall.py
@Path    :   agrothon/tgbot/modules/
@Time    :   2021/05/9
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.5
@Contact :   ckvbalusu@gmail.com
@Desc    :   Rainfall prediction module for Telegram bot
"""
from prettytable import PrettyTable

from agrothon import DISTRICT, LANG, STATE

from ..helpers.apiserverhelper import get_rainfall_prediction

MONTHS = [
    "april",
    "may",
    "june",
    "july",
    "august",
    "september",
    "october",
    "november",
    "december",
]


async def rainfall_predict(client, message):
    data = await get_rainfall_prediction(STATE, DISTRICT)
    if data is not None:
        units = data["units"]
        predictions = data["predictions"]
        pt = PrettyTable([LANG.MONTH, LANG.RAINFALL.format(units)])
        pt.align[LANG.MONTH] = "l"
        pt.align[LANG.RAINFALL.format(units)] = "c"
        pt.padding_width = 0
        i = 0
        for month in MONTHS:
            pt.add_row([LANG.MONTHS[i], predictions[month]])
            i += 1
        await message.reply_text(
            text=LANG.RAIN_PREDICT.format(STATE, DISTRICT, pt), parse_mode="HTML"
        )
    else:
        await message.reply_text(text=LANG.RAIN_PREDICT_ERR)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   API.py
@Path    :   agrothon/
@Time    :   2021/05/8
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.0
@Contact :   ckvbalusu@gmail.com
@Desc    :   Main module which starts API Server
"""
import logging
import os
import time

from fastapi import FastAPI, Request, Security
from fastapi.responses import RedirectResponse

from agrothon import __VERSION__, TIME_Z, MDBClient

from .server.helpers.api_key_helper import verify_api_key
from .server.routers import intruder, openweather, pump, rain_predict, sensors

logging.getLogger("pyrogram").setLevel(logging.WARNING)
logging.getLogger("urllib3").setLevel(logging.WARNING)
logging.getLogger("multpart").setLevel(logging.WARNING)

os.environ["TZ"] = TIME_Z
time.tzset()


Agrothon = FastAPI(
    title="Agrothon",
    version=__VERSION__,
    docs_url="/docs",
    redoc_url=None,
    openapi_url="/Agrothon.json",
    dependencies=[Security(verify_api_key)],
)


@Agrothon.middleware("http")
async def add_process_time_header(request: Request, call_next):
    start_time = time.time()
    response = await call_next(request)
    process_time = time.time() - start_time
    response.headers["X-Process-Time"] = str(round(process_time * 1000, 3)) + " ms"
    return response


@Agrothon.get("/ping", tags=["utils"])
async def ping():
    return {"Ping": "Pong"}


@Agrothon.get("/", response_class=RedirectResponse, include_in_schema=False)
async def root():
    return RedirectResponse("https://github.com/viswanathbalusu/agrothon")


Agrothon.add_event_handler("startup", MDBClient.connect_db)
Agrothon.add_event_handler("shutdown", MDBClient.close_mongo_connection)
Agrothon.include_router(intruder.IntruderRouter)
Agrothon.include_router(openweather.OpenWeatherRouter)
Agrothon.include_router(pump.PumpRouter)
Agrothon.include_router(sensors.SensorRouter)
Agrothon.include_router(rain_predict.RainPredictRouter)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   keyboards.py
@Path    :   agrothon/tgbot/helpers/
@Time    :   2021/05/8
@Author  :   Chandra Kiran Viswanath Balusu
@Version :   1.1.0
@Contact :   ckvbalusu@gmail.com
@Desc    :   Keyboards used in the telegram bot
"""

from pyrogram.types import InlineKeyboardButton, InlineKeyboardMarkup

from agrothon import LANG

fieldkey = InlineKeyboardMarkup(
    [
        [
            InlineKeyboardButton("💧 " + LANG.MOISTURE, callback_data="moisture"),
            InlineKeyboardButton("⛅ " + LANG.HUMIDITY, callback_data="humidity"),
        ],
        [
            InlineKeyboardButton("🌡️ " + LANG.TEMPERATURE, callback_data="temperature"),
            InlineKeyboardButton("✅ " + LANG.COMPLETE_INFO, callback_data="complete"),
        ],
        [InlineKeyboardButton("🚰 " + LANG.PUMP_STATUS, callback_data="pumpstat")],
        [InlineKeyboardButton("🛑 " + LANG.QUIT, callback_data="exit")],
    ]
)


sepkeyboard = InlineKeyboardMarkup(
    [
        [InlineKeyboardButton("🚰 " + LANG.PUMP_STATUS, callback_data="pumpstat")],
        [InlineKeyboardButton("⬅️ " + LANG.BACK, callback_data="back")],
    ]
)

pumpoffkey = InlineKeyboardMarkup(
    [
        [
            InlineKeyboardButton("🛑 " + LANG.PUMP_OFF, callback_data="pumpoff"),
        ],
        [InlineKeyboardButton("⬅️ " + LANG.BACK, callback_data="pumpstat")],
    ]
)

pumponkey = InlineKeyboardMarkup(
    [
        [
            InlineKeyboardButton("✅ " + LANG.PUMP_ON, callback_data="pumpon"),
        ],
        [InlineKeyboardButton("⬅️ " + LANG.BACK, callback_data="pumpstat")],
    ]
)


pumponmenu = InlineKeyboardMarkup(
    [
        [
            InlineKeyboardButton("🛑 " + LANG.PUMP_OFF, callback_data="pumpoff"),
            InlineKeyboardButton("🔄 " + LANG.REFRESH, callback_data="refresh"),
        ],
        [
            InlineKeyboardButton("🤖 " + LANG.BOT_PRED, callback_data="bot"),
        ],
        [InlineKeyboardButton("⬅️ " + LANG.BACK, callback_data="back")],
    ]
)

pumpoffmenu = InlineKeyboardMarkup(
    [
        [
            InlineKeyboardButton("✅ " + LANG.PUMP_ON, callback_data="pumpon"),
            InlineKeyboardButton("🔄 " + LANG.REFRESH, callback_data="refresh"),
        ],
        [
            InlineKeyboardButton("🤖 " + LANG.BOT_PRED, callback_data="bot"),
        ],
        [InlineKeyboardButton("⬅️ " + LANG.BACK, callback_data="back")],
    ]
)


def backkey(callback):
    return InlineKeyboardMarkup(
        [[InlineKeyboardButton("⬅️ " + LANG.BACK, callback_data=callback)]]
    )


settingskey = InlineKeyboardMarkup(
    [
        [InlineKeyboardButton(LANG.LANG, callback_data="lang")],
        [InlineKeyboardButton("🔄 " + LANG.RESTART_CALLBACK, callback_data="restart")],
        [InlineKeyboardButton("🛑 " + LANG.QUIT, callback_data="exit")],
    ]
)

languageskey = InlineKeyboardMarkup(
    [
        [
            InlineKeyboardButton("English ", callback_data="eng"),
            InlineKeyboardButton("తెలుగు", callback_data="tel"),
        ],
        [
            InlineKeyboardButton("தமிழ்", callback_data="tam"),
            InlineKeyboardButton("हिन्दी", callback_data="hin"),
        ],
        [InlineKeyboardButton("🛑 " + LANG.QUIT, callback_data="exit")],
    ]
)
